--------------------------------------------------------------------------
-- Assirc_Database                          Copyrigth ENS ULM 1996-1998 --
-- Pierre VINCENS                   V 00.000                 10/04/1996 --
-- Pierre VINCENS                   V 01.000                 31/03/1998 --
--------------------------------------------------------------------------
with Ada.Unchecked_Deallocation;
with Ada.Text_Io; use Ada.Text_Io;
with Type_Specifications.IO; use Type_Specifications.IO;
with Strlib; use Strlib;
with Options_Util; use Options_Util;
  
with Sequence_Specifications; use Sequence_Specifications;
with Sequence_Specifications.Basic; use Sequence_Specifications.Basic;
with Assirc_Specifications.Database; use Assirc_Specifications.Database;
with Assirc_Global_Variables; use Assirc_Global_Variables;
with Assirc_Database_Functions; use Assirc_Database_Functions;

package body Assirc_Database is

--------------------------------------------------------------------------
-- Acces aux bibliothques                                              --
--------------------------------------------------------------------------
  use Type_Specifications.IO.Int_Io;
  use Type_Specifications.IO.Real_Io;
  use Type_Specifications.Real_Lib;
  
  package SRRBT renames Assirc_Specifications.Database.Similar_Regions_RB_Tree;
  use SRRBT; 

--------------------------------------------------------------------------
-- Positionner les paramtres de gestion                                --
--------------------------------------------------------------------------
  procedure Set_Parameters(Excluded_Marge     : in Integer;
			   Intersection_Marge : in Integer) is

  begin
    Excluded := Excluded_Marge;
    Intersection := Intersection_Marge;
  end Set_Parameters;
  
--------------------------------------------------------------------------
-- Dterminer si un couple de segments est dj inclus dans la base     --
--------------------------------------------------------------------------
  function Included(Seg1,Seg2 : in Segment_Type) return Boolean is
  begin
    return Included(Seg1,Seg2,Similar_Regions,Excluded);
  end Included;
  

--------------------------------------------------------------------------
-- Ajouter une paire dans l'arbre                                       --
--------------------------------------------------------------------------
  procedure Append_Pair_In_Tree(Pair : in out Segment_Pair) is
  begin
    Append_Pair_In_Tree(Similar_Regions,Pair,Intersection);
  end Append_Pair_In_Tree; 
  
--------------------------------------------------------------------------
-- Impression des rsultats                                             --
--------------------------------------------------------------------------
  procedure Print_Database(Name           : in String;
			   Minimal_Length : in Integer) is
    -- Le contenu de cette procedure est repris dans dassirc_dispatcher
    -- Les modifications eventuelles doivent y etre reportees. Le fait
    -- que les pointeurs de fichiers soient limites prives impose 
    -- cette contrainte

    Suffix : constant String := ".sil";
    File : aliased Ada.Text_Io.File_Type;
    -- Le pointeur vers le fichier
    Output : Ada.Text_Io.File_Access;
    -- et un accs  ce pointeur
    
    procedure Print_A_Pair(Pair   : in out Segment_Pair;
			   Output : in out Ada.Text_Io.File_Access) is
      File : Ada.Text_Io.File_Type renames Output.all; 
    begin
      if Pair.Length < Minimal_Length then return; end if;
      Put(File,To_String(Sequences(Pair.Seg1.Id).Identifier));
      Set_Col(File,12);
      Put(File,Image(Pair.Seg1.Strand));
      if Pair.Seg1.Strand = Watson then
	Put(File,Pair.Seg1.Start,10);
	Put(File,Pair.Seg1.Stop,10);
      else -- Crick on inscrit desormais dans le meme ordre
	Put(File,Pair.Seg1.Start,10);
      	Put(File,Pair.Seg1.Stop,10);
      end if;
      Put(File," ");
      Put(File,To_String(Sequences(Pair.Seg2.Id).Identifier));
      Set_Col(File,45);
      Put(File,Image(Pair.Seg2.Strand));
      if Pair.Seg2.Strand = Watson then
	Put(File,Pair.Seg2.Start,10);
	Put(File,Pair.Seg2.Stop,10);
      else -- Crick on inscrit desormais dans le meme ordre
	Put(File,Pair.Seg2.Start,10); 
      	Put(File,Pair.Seg2.Stop,10);
      end if;
      Put(File,Pair.Length,10);
      Put(File,Sqrt(Real(Pair.Distance)),8,3,0);
      New_Line(File);
    end Print_A_Pair;
    procedure Print_Pairs is new SRRBT.Apply_With_Data(Ada.Text_Io.File_Access,
						       Print_A_Pair);

 begin 
    -- ouvre un fichier name avec terminaison ".sim"
    if Name'Length > 5 and then Name(Name'Last-4..Name'Last) = Suffix then
      Create(File,Out_File,Name);
    else
      Create(File,Out_File,Name & Suffix);
    end if;
    Output := File'Unchecked_Access; 
    -- Le Unchecked est requis car un File_Type est limit et faire
    -- cette opration donne la possibilit de realiser une affectation
    -- dessus. La dfinition de Rb_Tree.Apply_With_Data suppose que le
    -- second parametre est pass en in_out. Il doit donc etre donne
    -- comme une variable, d'ou la ncessit de passer par le File_Access
    -- En pratique, ce parametre est utilise en mode In seulement. En 
    -- attendant d'ecrire une variante de Apply_With_Data prenant des
    -- parametres de mode In, on utilise cette mthode peu lgante.
    -- Note supplmentaire: une premire version o le parametre File
    -- tait global  la procdure print_database provoquait une erreur
    -- du linker. Il n'a pas ete possible de localiser preciserment
    -- la cause de cette erreur (Point d'entree sur une instance
    -- d'un generique non trouv). 
    Print_Pairs(Similar_Regions.Tree,Output);
    Close(File);
  exception
    when Name_Error =>
      -- si cela n'est pas possible, on ouvre le fichier dans /tmp
      declare
	New_Name : constant String := "/tmp/" & 
	  Root_Name(To_Varstring(Name)) & Suffix;
      begin
	Put_Line("WARNING: Output data are write in " & New_Name);
	Create(File,Out_File,New_Name);
	Output := File'Unchecked_Access;
	-- Mme remarque que ci-dessus
	Print_Pairs(Similar_Regions.Tree,Output);
	Close(File); 
      exception
	when Name_Error =>
	  -- et sinon on ecrit sur la sortie standard
	  Put_Line("WARNING: Output data are write to standard output");
	  Output := Standard_Output;
	  Print_Pairs(Similar_Regions.Tree,Output);
      end;
  end Print_Database;

--------------------------------------------------------------------------
-- Rinitialisation de la base de donnes                               --
--------------------------------------------------------------------------
  procedure Reset_Tree is
  begin
    Reset(Similar_Regions.Tree);
    Similar_Regions.Max_Seg_Length := 0;
  end Reset_Tree;

--------------------------------------------------------------------------
-- Applications d'une procdure  tous les lments de la base          --
--------------------------------------------------------------------------
  procedure Apply_To_Tree is
    procedure Command_For_All is new SRRBT.Apply(Command);
  begin
    Command_For_All(Similar_Regions.Tree);
  end Apply_To_Tree;

end Assirc_Database;
