--------------------------------------------------------------------------
-- Assirc_Seeds.D_Eval_M1                   Copyrigth ENS ULM 1996-2000 --
-- Pierre VINCENS                   V 00.000                 10/04/1996 --
-- Pierre VINCENS                   V 01.000                 23/03/1998 --
-- Pierre VINCENS                   V 01.001                 17/04/1998 --
-- Pierre VINCENS                   V 01.002                 27/12/2000 --
--------------------------------------------------------------------------
-- REMARQUE:
-- Pour protger l'accs concurrent ventuel  dassirc_database et 
-- dassirc_dispatcher, tous les appels sont protgs par un mcanisme de 
-- smaphore
-- v 01.002 : un mecanisme de controle est ajoute pour tenir compte
-- des equivalences W:a-b C:c-d avec C:a-b et W:b-c
--------------------------------------------------------------------------
with Strlib; use Strlib;

with Assirc_Statistics; use Assirc_Statistics;

with Assirc_Database; 
with Dassirc_Database; 

with Dassirc_Semaphore; use Dassirc_Semaphore;

package body Assirc_Seeds.D_Eval_M1 is

--------------------------------------------------------------------------
-- Acces aux bibliothques                                              --
--------------------------------------------------------------------------
  package SL renames Assirc_Specifications.Database.Segment_List;
  use SL;
  
--------------------------------------------------------------------------
-- Gestion des paires de brins a analyser                               --
--------------------------------------------------------------------------
  Pairs_To_Check : Access_To_Array_Pairs_To_Check;
  procedure Reset_Pairs_To_Check is
  begin
    Free(Pairs_To_Check);
  end Reset_Pairs_To_Check;
  
--------------------------------------------------------------------------
-- Evaluation des graines                                               --
--------------------------------------------------------------------------
  procedure Eval_Seeds(Sequences         : in Assirc_Sequence_Array_Access;
                       Seeds             : in out Seed_Segments;
                       Min_Length        : in Integer;
                       Max_Dist          : in Integer;
                       Tolerance         : in Integer;
                       Common_Max_Dist   : in Integer;
                       Skip              : in Integer;
                       Xonly             : in Boolean;
		       Zone1,Zone2       : in Segment_Type;
		       Z1_Xonly,Z2_Xonly : in Boolean) is
    -- Zone1 et Zone2 definissent l'intervalle dans lesquelles
    -- sont recherches les duplications
    -- Z?_Xonly indique si l'on doit ou non faire une recherche
    -- de la zone contre elle-meme
    use Assirc_Database;
    Max_Dist2 : constant Integer := Max_Dist * Max_Dist; 
    
    function Is_Into(Seg  : in Segment_Type;
		     Zone : in Segment_Type) return Boolean;
    pragma Inline(Is_Into);
    function Is_Into(Seg  : in Segment_Type;
		     Zone : in Segment_Type) return Boolean is
      -- Indique que le segment seg se trouve  l'intrieur de la zone
    begin
      if Seg.Id /= Zone.Id then return False; end if;
      return (Seg.Start >= Zone.Start and 
	      Seg.Start <= Zone.Stop);
    end Is_Into;
      
    procedure Eval_A_Seed(Seed : in out Segment_List.List_Handler) is
      -- cette procedure genere tous les couples associes a une graine
      -- et les evalue
      Entries : SL.List_Entries;
      Pair    : Segment_Pair;
    begin
      -- transformer la liste des segments en tableau
      Get_Entries(Seed,Entries);
      -- generer la liste des couples
      for I in Entries'Range loop
        for J in I+1..Entries'Last loop
          -- Constitution d'un nouveau couple
	  Statistics.Increment_Constituted;
          -- On ne traite que les paires prevues
          if Pairs_To_Check(Consult(Entries(I)).Id,Consult(Entries(I)).Strand,
 			    Consult(Entries(J)).Id,Consult(Entries(J)).Strand)
	    and then
	    (not (Z1_Xonly and then
		  Is_Into(Consult(Entries(I)),Zone1) and then
		  Is_Into(Consult(Entries(J)),Zone1)))
	    and then
	    (not (Z2_Xonly and then
		  Is_Into(Consult(Entries(I)),Zone2) and then
		  Is_Into(Consult(Entries(J)),Zone2)))
	  then
	    declare
	      Seg1 : Segment_Type := Consult(Entries(I));
	      Seg2 : Segment_Type := Consult(Entries(J));
	      Segb : Segment_Type;
	    begin
	      -- Dans le cas de segments situes sur un meme chromosome
	      -- et sur des brins differents, on determine le couple 
	      -- d'indice le plus bas
	      if Seg1.Id = Seg2.Id and Seg1.Strand /= Seg2.Strand then
		if Seg1.Strand > Seg2.Strand then
		  -- on inverse les deux segments pour que le brin 
		  -- Crick soit en premier
		  Segb := Seg1;
		  Seg1 := Seg2;
		  Seg2 := Segb;
		end if;
		if Seg1.Start > Seg2.Start then
		  -- on inverse les deux positions car  W:a-b et C:c-d
		  -- est equivalent a C:a-b avec W:c-d
		  Segb := Seg1;
		  Seg1.Start := Seg2.Start;
		  Seg1.Stop  := Seg2.Stop;
		  Seg2.Start := Segb.Start;  
		  Seg2.Stop  := Segb.Stop;  
		end if;
	      end if;
	      -- Pour chaque couple, le segment d'indice plus bas est mis
	      -- en premier
	      if Seg1.Id < Seg2.Id 
		or else
		(Seg1.Id = Seg2.Id and
		 (Seg1.Strand < Seg2.Strand
		  or else
		(Seg1.Strand = Seg2.Strand and
		 Seg1.Start  < Seg2.Start)))
	      then
		null;
	      else
		Segb := Seg1;
		Seg1 := Seg2;
		Seg2 := Segb;
	      end if;
	      -- pour chaque couple, verifier s'il est deja inclus dans un 
	      -- segment et qu'il n'est pas superpose. Si Xonly est vrai on 
	      -- ne teste que les fragments entre deux sequences differentes
	      if (not (Xonly and Seg1.Id = Seg2.Id) and then
		not Common_Origin(Seg1,Seg2,Common_Max_Dist) and then
		not Included(Seg1,Seg2))
	      then
		-- Il est dans une region a tester
		Statistics.Increment_Tested;
		-- pour chaque couple non inclus, verifier
		-- s'il y a une zone d'homologie
		Check_A_Pair(Seg1,Seg2,Sequences,
			     Max_Dist2,Tolerance,Skip,Pair);
		-- verifier si le couple satisfait aux exigences de longueur
		-- et de distance
		if Pair.Distance <= Max_Dist2 and Pair.Length >= Min_Length and
		  not Common_Origin(Pair.Seg1,Pair.Seg2,Common_Max_Dist) then
		  -- enregistrer le couple
		  Statistics.Increment_Registered;
		  Append_Pair_In_Tree(Pair);
		end if;
	      end if;
	    end;
	  end if;
	end loop;
      end loop;
      -- liberer la place associee au tableau
      Free(Entries);
    end Eval_A_Seed;

  begin -- Eval_Seeds
    if Pairs_To_Check = null then
      -- On ne fait ce calcul qu'au premier appel ou apres un reset
      Pairs_To_Check := Build_Array_Of_Pairs_To_Check(Sequences);
    end if;
    for I in Seeds'Range loop
      Eval_A_Seed(Seeds(I));
    end loop;
  end Eval_Seeds;

  procedure Send_Pairs(Minimal_Length : in Integer) is
    Max_Pairs : constant Integer := Dassirc_Database.Max_Pairs_For_Diffusion;
    Pairs : Segment_Pairs(1..Max_Pairs);
    Current : Integer := Pairs'First;
    procedure Collect_A_Pair(Pair : in out Segment_Pair) is
    begin
      if Pair.Length < Minimal_Length then return; end if;
      Pairs(Current) := Pair;
      if Current = Pairs'Last then
        -- Tableau plein, on poste
	Database_Access.Lock;
        Dassirc_Database.Append_Pairs_In_Tree(Pairs);
	Database_Access.Unlock;
        Current := Pairs'First;
      else
        Current := Current + 1;
      end if;
    end Collect_A_Pair;
    procedure Collect_Pairs is new 
      Assirc_Database.Apply_To_Tree(Collect_A_Pair);
  begin
    Collect_Pairs;
    -- Il reste  poster le dernier tableau
    if Current > Pairs'First then
      Database_Access.Lock;
      Dassirc_Database.Append_Pairs_In_Tree(Pairs(Pairs'First..Current-1));
      Database_Access.Unlock;       
    end if;
    -- Et on purge la base
    Assirc_Database.Reset_Tree;
  end Send_Pairs;

end Assirc_Seeds.D_Eval_M1;
