--------------------------------------------------------------------------
-- Assirc_Seeds                             Copyrigth ENS ULM 1996-1998 --
-- Pierre VINCENS                   V 00.000                 10/04/1996 --
-- Pierre VINCENS                   V 01.000                 24/07/1997 --
-- Pierre VINCENS                   V 01.001                 23/03/1998 --
-- Pierre VINCENS                   V 01.002                 23/03/1998 --
--------------------------------------------------------------------------
with Strlib; use Strlib;

with Assirc_Statistics; use Assirc_Statistics;

with Assirc_Database; use Assirc_Database;

package body Assirc_Seeds.Eval is

--------------------------------------------------------------------------
-- Acces aux bibliothques                                              --
--------------------------------------------------------------------------
  package SL renames Assirc_Specifications.Database.Segment_List;
  use SL;

--------------------------------------------------------------------------
-- Evaluation des graines                                               --
--------------------------------------------------------------------------
  procedure Eval_Seeds(Sequences       : in Assirc_Sequence_Array_Access;
                       Seeds           : in out Seed_Segments;
                       Min_Length      : in Integer;
                       Max_Dist        : in Integer;
                       Tolerance       : in Integer;
                       Common_Max_Dist : in Integer;
                       Skip            : in Integer;
                       Xonly           : in Boolean) is
    Max_Dist2 : constant Integer := Max_Dist * Max_Dist;
    Pairs_To_Check : Access_To_Array_Pairs_To_Check :=
      Build_Array_Of_Pairs_To_Check(Sequences);

    procedure Eval_A_Seed(Seed   : in out Segment_List.List_Handler) is
      -- cette procedure genere tous les couples associes a une graine
      -- et les evalue
      Entries : SL.List_Entries;
      Pair    : Segment_Pair;
    begin
      -- transformer la liste des segments en tableau
      Get_Entries(Seed,Entries);
      -- generer la liste des couples
      for I in Entries'Range loop
        for J in I+1..Entries'Last loop
          -- Constitution d'un nouveau couple
	  Statistics.Increment_Constituted;
	  -- On ne traite que les paires prevues
	  if Pairs_To_Check(Consult(Entries(I)).Id,Consult(Entries(I)).Strand,
			    Consult(Entries(J)).Id,Consult(Entries(J)).Strand)
	  then
            declare
	      Seg1 : Segment_Type := Consult(Entries(I));
	      Seg2 : Segment_Type := Consult(Entries(J));
	      Segb : Segment_Type;
	    begin
	      -- Dans le cas de segments situes sur un meme chromosome
	      -- et sur des brins differents, on determine le couple 
	      -- d'indice le plus bas
	      if Seg1.Id = Seg2.Id and Seg1.Strand /= Seg2.Strand then
		if Seg1.Strand > Seg2.Strand then
		  -- on inverse les deux segments pour que le brin 
		  -- Crick soit en premier
		  Segb := Seg1;
		  Seg1 := Seg2;
		  Seg2 := Segb;
		end if;
		if Seg1.Start > Seg2.Start then
		  -- on inverse les deux positions car  W:a-b et C:c-d
		  -- est equivalent a C:a-b avec W:c-d
		  Segb := Seg1;
		  Seg1.Start := Seg2.Start;
		  Seg1.Stop  := Seg2.Stop;
		  Seg2.Start := Segb.Start;  
		  Seg2.Stop  := Segb.Stop;  
		end if;
	      end if;
	      -- Pour chaque couple, le segment d'indice plus bas est mis
	      -- en premier
	      if Seg1.Id < Seg2.Id 
		or else
		(Seg1.Id = Seg2.Id and
		 (Seg1.Strand < Seg2.Strand
		  or else
		(Seg1.Strand = Seg2.Strand and
		 Seg1.Start  < Seg2.Start)))
	      then
		null;
	      else
		Segb := Seg1;
		Seg1 := Seg2;
		Seg2 := Segb;
	      end if;
	      -- pour chaque couple, verifier s'il est deja inclus dans un 
	      -- segment et qu'il n'est pas superpose. Si Xonly est vrai on 
	      -- ne teste que les fragments entre deux sequences differentes
	      if (not (Xonly and Seg1.Id = Seg2.Id) and then
		not Common_Origin(Seg1,Seg2,Common_Max_Dist) and then
		not Included(Seg1,Seg2))
	      then
		-- Il est dans une region a tester
		Statistics.Increment_Tested;
		-- pour chaque couple non inclus, verifier
		-- s'il y a une zone d'homologie
		Check_A_Pair(Seg1,Seg2,Sequences,
			     Max_Dist2,Tolerance,Skip,Pair);
		-- verifier si le couple satisfait aux exigences de longueur
		-- et de distance
		if Pair.Distance <= Max_Dist2 and 
		  Pair.Length >= Min_Length and
		  not Common_Origin(Pair.Seg1,Pair.Seg2,Common_Max_Dist) then
		  -- enregistrer le couple
		  Statistics.Increment_Registered;
		  Append_Pair_In_Tree(Pair);
		end if;
	      end if;
	    end;
          end if;
        end loop;
      end loop;
      -- liberer la place associee au tableau
      Free(Entries);
    end Eval_A_Seed;

  begin -- Eval_Seeds
    for I in Seeds'Range loop
      Eval_A_Seed(Seeds(I));
    end loop;
    Free(Pairs_To_Check);
  end Eval_Seeds;

end Assirc_Seeds.Eval;
