--------------------------------------------------------------------------
-- Assirc_Seeds                             Copyrigth ENS ULM 1996-1998 --
-- Pierre VINCENS                   V 00.000                 10/04/1996 --
-- Pierre VINCENS                   V 01.000                 24/07/1997 --
-- Pierre VINCENS                   V 01.001                 23/03/1998 --
-- Pierre VINCENS                   V 01.002                 23/03/1998 --
--------------------------------------------------------------------------
with Ada.Unchecked_Conversion;
with Ada.Text_Io; use  Ada.Text_Io;
with Type_Specifications.IO; use Type_Specifications.IO;
with Strlib; use Strlib;

with Sequence_Specifications.Basic; use Sequence_Specifications.Basic;
with Sequence_Io; use Sequence_Io;

with Assirc_Seeds_Util; use Assirc_Seeds_Util;

package body Assirc_Seeds is

--------------------------------------------------------------------------
-- Acces aux bibliothques                                              --
--------------------------------------------------------------------------
  use Type_Specifications.IO.Int_Io;
  use Type_Specifications.IO.Real_Io;
  use Type_Specifications.Real_Lib;

  package SL renames Assirc_Specifications.Database.Segment_List;
  use SL;

--------------------------------------------------------------------------
-- Gestion des parametres                                               --
--------------------------------------------------------------------------
  type Shift_Array is array(Watson..Crick) of Shift_Access;
  type Seeds_Parameters is record
    Shift : Shift_Array;
  end record;
  Params : Seeds_Parameters;

  procedure Set_Seed_Parameters(Shift : Shift_Type) is
  begin
    -- initialisation de params.shift a la bonne taille
    Params.Shift(Watson) := new Shift_Type(Shift'Range(1),Shift'Range(2));
    Params.Shift(Crick)  := new Shift_Type(Shift'Range(1),Shift'Range(2));
    -- Pour initialiser par defaut
    Params.Shift(Watson).all := Shift;
    Params.Shift(Crick).all := Shift; 
    -- et apres les bonnes valeurs
    for I in Shift'Range(2) loop
      Params.Shift(Crick)('A',I) := Shift('T',I); 
      Params.Shift(Crick)('T',I) := Shift('A',I);
      Params.Shift(Crick)('G',I) := Shift('C',I);
      Params.Shift(Crick)('C',I) := Shift('G',I);
    end loop;
  end Set_Seed_Parameters;

--------------------------------------------------------------------------
-- Quelques fonctions utilitaires internes                              --
--------------------------------------------------------------------------
  function Common_Origin(Seg1,Seg2 : in Segment_Type;
                         Max_Dist  : in Integer) return boolean is
  begin
    if Seg1.Id = Seg2.Id and Seg1.Strand = Seg2.Strand then
      return abs(Seg1.Start - Seg2.Start) < Max_Dist;
    end if;
    return False;
  end Common_Origin;

 

--------------------------------------------------------------------------
-- Test de l'adquation de deux segments  tre homologue               --
--------------------------------------------------------------------------
  procedure Check_A_Pair(Seg1,Seg2 : in Segment_Type;
                         Sequence  : in Assirc_Sequence_Array_Access;
                         Max_Dist2 : in Integer;
                         Tolerance : in Integer;
                         Skip      : in Integer;
                         Pair      : in out Segment_Pair) is
    --type Vector is array(Params.Shift(Watson)'Range(2)) of Integer;
      -- vecteur dcrivant la position sur le chemin de la marche alatoire

    Seq1    : Nucleotidic_Sequence renames Sequence(Seg1.Id).Nucleotides;
    Seq2    : Nucleotidic_Sequence renames Sequence(Seg2.Id).Nucleotides;
    Inf1,Sup1 : Integer; -- indice segment1
    Inf2,Sup2 : Integer; -- indice segment2
    Increment1,Increment2 : Integer; -- Deplacement sur chaque sequence
    ID2,SD2 : Integer;
      -- distance calcule pour le segment suprieur et infrieur

    procedure Check(Pos1,Pos2  : in out Integer;
                    D2         : out Integer;
                    Increment1 : in Integer;
                    Increment2 : in Integer);
    pragma Inline(Check);
    procedure Check(Pos1,Pos2  : in out Integer;
                    D2         : out Integer;
                    Increment1 : in Integer;
                    Increment2 : in Integer) is
      -- Cette procedure determine la position ou la distance entre
      -- les marches aleatoires de chacune des deux squences dpasse
      -- le seuil fix max_dist2 (carr de la distance max).
      -- Attention, afin de limiter les calculs l'ecart de distance n'est
      -- calcul que tous les skip nuclotides. La position sera
      -- donc donne par dfaut. Il faut tenir compte de cette valeur de
      -- skip dans la valeur effective de la tolrance
      V1,V2 : Vector(Params.Shift(Watson)'Range(2)) := (others => 0);
      --X1,Y1,X2,Y2 : Integer := 0;
        -- position courante de la marche
      OD2,RD2 : Integer := 0;
        -- distance prcdente et actuelle entre les deux marches
      Lpos1   : Integer := Pos1;
      Lpos2   : Integer := Pos2;
      LD2     : Integer := 0;
        -- mmorisation des dernires positions valides
      Ret   : Integer := 0;
        -- nombre de progressions valides
      Count : Integer := 0;     

    begin
      Progression: loop
        Count := Count + 1;
        Pos1 := Pos1 + Increment1;
        Pos2 := Pos2 + Increment2;
        if (Pos1 < Seq1'First or  Pos2 < Seq2'First or
            Pos1 > Seq1'Last or Pos2 > Seq2'Last) then
          Lpos1 := Pos1 - Increment1;
          Lpos2 := Pos2 - Increment2;
          LD2 := OD2;
          exit;
        end if;
        -- on saute les positions comportant des nuclotides indtermins
        --if Seq1(Pos1) /= 'N' and Seq2(Pos2) /= 'N' then
	if Seq1(Pos1) in 'A'..'C' and Seq2(Pos2) in 'A'..'C' then
          New_Shift(Seq1(Pos1),Params.Shift(Seg1.Strand).all,V1);
          New_Shift(Seq2(Pos2),Params.Shift(Seg2.Strand).all,V2);
	  if Ret = 0 then
            if Count >= Skip then
              -- on reinitialise count a 0. On procede ainsi afin de
              -- prendre en compte les eventuels nucleotides indtermins
              Count := 0;
              -- calculer la distance entre V1 et V2
              RD2 := Distance_2(V1,V2);
              -- on declenche la prise en compte de la zone de tolerance
              if RD2 > Max_Dist2 then
                Lpos1 := Pos1 - Increment1;
                Lpos2 := Pos2 - Increment2;
                LD2 := OD2;
                Ret := 1;
              end if;
            end if;
          else
            -- calculer la distance entre V1 et V2
            RD2 := Distance_2(V1,V2);
            -- on regarde si on repasse en dessous du seuil
            if RD2 < Max_Dist2 then
              Ret := 0;
              Count := 0;
            else
              Ret := Ret + 1;
              exit Progression when Ret >= Tolerance;
                -- on a atteind la limite
            end if;
          end if;
          -- memoriser la derniere distance en cas de franchissement
          -- de la limite
          OD2 := RD2;
        end if;
      end loop Progression;
      Pos1 := Lpos1;
      Pos2 := Lpos2;
      D2 := LD2;
      --Put_Line("Fin: " & Integer'Image(D2));
    end Check;

  begin
    -- En fonction des brins sur lesquels on se trouve, determiner les points
    -- de depart
    if Seg1.Strand = Watson then
      Inf1 := Seg1.Start; -- indice segment1
      Sup1 := Seg1.Stop;  -- indice segment1
      Increment1 := 1;
    else
      Inf1 := Seg1.Stop;  -- indice segment1
      Sup1 := Seg1.Start; -- indice segment1
      Increment1 := -1;
    end if;
    if Seg2.Strand = Watson then
      Inf2 := Seg2.Start; -- indice segment2
      Sup2 := Seg2.Stop;  -- indice segment2
      Increment2 := 1;
    else
      Inf2 := Seg2.Stop; -- indice segment2
      Sup2 := Seg2.Start;  -- indice segment2
      Increment2 := -1;
    end if;
         
    -- marche en avant pour definir la borne superieure
    Check(Sup1,Sup2,SD2,Increment1,Increment2);
    -- marche en arriere pour definir la borne inferieure
    Check(Inf1,Inf2,ID2,-Increment1,-Increment2);
    -- On positionne Start < Stop
    if Inf1 > Sup1 then Swap(Inf1,Sup1); end if;
    if Inf2 > Sup2 then Swap(Inf2,Sup2); end if;
    -- si les deux segments sont de la meme sequence alors on les
    -- ordonne
    if Seg1.Id = Seg2.Id and Seg1.Strand = Seg2.Strand and Inf1 > Inf2 then
      Swap(Inf1,Inf2);
      Swap(Sup1,Sup2);
    end if;
    -- memoriser les donnees
    Pair.Seg1.Id     := Seg1.Id;
    Pair.Seg1.Strand := Seg1.Strand;
    Pair.Seg1.Start  := Inf1;
    Pair.Seg1.Stop   := Sup1;
    Pair.Seg2.Id     := Seg2.Id;
    Pair.Seg2.Strand := Seg2.Strand;
    Pair.Seg2.Start  := Inf2;
    Pair.Seg2.Stop   := Sup2;
    -- on prend la moyenne des distances finales
    Pair.Distance   := (SD2+ID2)/2;
    -- la longueur est obligatoirement la meme sur les deux segment
    -- compte-tenu de la maniere dont se fait la construction
    Pair.Length := Sup1 - Inf1 + 1;
  end Check_A_Pair;

--------------------------------------------------------------------------
-- Determination des couples Watson-Crick a evaluer                     --
--------------------------------------------------------------------------
  function Build_Array_Of_Pairs_To_Check
    (Sequences : in Assirc_Sequence_Array_Access)
    return Access_To_Array_Pairs_To_Check is
    Pairs_To_Check : Access_To_Array_Pairs_To_Check :=
      new Array_Of_Pairs_To_Check(Sequences'Range,Watson..Crick,
                                  Sequences'Range,Watson..Crick);
  begin
    Pairs_To_Check.all := (others=>(others=>(others=>(others=>False))));
    for I in Pairs_To_Check'Range(1) loop
      for J in I..Pairs_To_Check'Last(3) loop
        case Sequences(I).Strand is
          when Watson =>
             case Sequences(J).Strand is
               when Watson => 
                 Pairs_To_Check(I,Watson,J,Watson) := True;
                 Pairs_To_Check(J,Watson,I,Watson) := True;
               when Crick  =>
                 Pairs_To_Check(I,Watson,J,Crick)  := True;
                 Pairs_To_Check(J,Crick,I,Watson)  := True;
               when Both   =>
                 Pairs_To_Check(I,Watson,J,Watson) := True;
                 Pairs_To_Check(J,Watson,I,Watson) := True;
                 Pairs_To_Check(I,Watson,J,Crick)  := True;
                 Pairs_To_Check(J,Crick,I,Watson)  := True;
             end case;
           when Crick =>
             case Sequences(J).Strand is
               when Watson => 
                 Pairs_To_Check(I,Crick,J,Watson)  := True;
                 Pairs_To_Check(J,Watson,I,Crick)  := True;
               when Crick  =>
                 Pairs_To_Check(I,Crick,J,Crick)   := True;
                 Pairs_To_Check(J,Crick,I,Crick)   := True;
               when Both   =>
                 Pairs_To_Check(I,Crick,J,Watson)  := True;
                 Pairs_To_Check(J,Watson,I,Crick)  := True;
                 Pairs_To_Check(I,Crick,J,Crick)   := True;
                 Pairs_To_Check(J,Crick,I,Crick)   := True;
             end case;
           when Both =>
             case Sequences(J).Strand is
               when Watson => 
                 Pairs_To_Check(I,Watson,J,Watson) := True;
                 Pairs_To_Check(J,Watson,I,Watson) := True;
                 Pairs_To_Check(I,Crick,J,Watson)  := True;
                 Pairs_To_Check(J,Watson,I,Crick)  := True;
               when Crick  =>
                 Pairs_To_Check(I,Watson,J,Crick)  := True;
                 Pairs_To_Check(J,Crick,I,Watson)  := True;
                 Pairs_To_Check(I,Crick,J,Crick)   := True;
                 Pairs_To_Check(J,Crick,I,Crick)   := True;
               when Both   =>
                 Pairs_To_Check(I,Watson,J,Watson) := True;
                 Pairs_To_Check(J,Watson,I,Watson) := True;
                 Pairs_To_Check(I,Watson,J,Crick)  := True;
                 Pairs_To_Check(J,Crick,I,Watson)  := True;
             end case;
         end case;
      end loop;
    end loop; 
    return Pairs_To_Check;
  end Build_Array_Of_Pairs_To_Check;
             
--------------------------------------------------------------------------
-- Gestion de la structure de donnes associe  la gestion des graines --
--------------------------------------------------------------------------
  procedure Init_Seeds(Seed_Size   : in Integer;
                       Seeds       : in out Seed_Segments) is
    Code_Length : constant Integer := (Nb_Symbol ** Seed_Size)-1;
  begin
    -- initialiser le tableau des graines
    Seeds := new Seed_Segment_Array(0..Code_Length);
  end Init_Seeds;

  procedure Reset_Seeds(Seeds : in out Seed_Segments) is
  begin
    if Seeds = null then return; end if;
    for I in Seeds'Range loop
      SL.Reset(Seeds(I));
    end loop;
  end Reset_Seeds;

  procedure Free_Seeds(Seeds : in out Seed_Segments) is
  begin
    Reset_Seeds(Seeds);
    Free(Seeds);
  end Free_Seeds;

--------------------------------------------------------------------------
-- Dtermination des graines sans utilisation d'une double indexation   --
--------------------------------------------------------------------------
  procedure Get_Seeds(Sequence    : in Assirc_Sequence_Data;
                      Strand      : in Strand_Type;
                      Seed_Size   : in Integer;
                      Seeds       : in out Seed_Segments)  is
    Seq   : Nucleotidic_Sequence renames Sequence.Nucleotides;
    type Adr_Array is array(Integer range <>) of Integer;
    Adr : Adr_Array(0..Seed_Size) := (others => 0);
    Code_Length : constant Integer := (Nb_Symbol ** Seed_Size)-1;
    Code_Size : constant Integer := Seed_Size + 1;
    Shift : constant Integer := Nb_Symbol ** (Seed_Size-1);
    Current_First : Integer := 0;
    Current_Last : Integer := 1;
    Code : Integer := 0;
    Segment : Segment_Type;
    From,To,Limit,Increment : Integer;
    I    : Integer;

  begin
    -- Pour tous les segments generes, l'identificateur sera le meme
    -- pour cette sequence
    Segment.Id := Sequence.Id;
    Segment.Strand := Strand;
    -- En fonction du brin considere, l'analyse se fait dans un sens ou 
    -- dans l'autre
    if Strand = Watson then
      From := Sequence.Start;
      To   := Sequence.Stop;
      Increment := 1;
    else
      From := Sequence.Stop;
      To   := Sequence.Start;
      Increment := -1;
    end if;
    -- Boucle sur la zone a etudier
    loop
      begin
        -- Construire le codage initial du premier segment
        Adr(0) := 0; -- cas du premier intervalle
        Current_First := 0;
        Current_Last := 1;
        Code := 0;
        I := From - Increment;
        Limit := From + Increment * (Seed_Size-2);
        while I /= Limit loop
          if I = To then return; end if;
          I := I + Increment;
          Adr(Current_Last) := Tnu(Strand,Seq(I));
          Code := Code * Nb_Symbol + Adr(Current_Last);
          Current_Last := Current_Last + 1;
        end loop;
        -- parcours de l'arbre
        loop
          if I = To then return; end if;
          I := I + Increment;
          -- determiner le code
          Adr(Current_Last) := Tnu(Strand,Seq(I));
          Code := ((Code - (Adr(Current_First)*Shift)) * Nb_Symbol) +
            Adr(Current_Last);
          -- calcul des indices pour le code
          Current_First := (Current_First + 1) mod Code_Size;
          Current_Last := (Current_Last + 1) mod Code_Size;
          -- enregistrer l'intervalle
          if Strand = Watson then
            Segment.Start := I - Seed_Size + 1;
            Segment.Stop  := I;
          else
            Segment.Start := I;
            Segment.Stop  := I + Seed_Size + 1;
          end if;
          Insert(Seeds(Code),Segment);                   
        end loop;
      exception
        when UNDEFINED_NUCLEOTIDE =>
          From := I + Increment;
      end;
      -- on quitte si from = to
      exit when From = To;
    end loop;
  end Get_Seeds;

--------------------------------------------------------------------------
-- Dtermination des graines avec double indexation                     --
--------------------------------------------------------------------------
  procedure Get_Seeds_By_Mask(Sequence    : in Assirc_Sequence_Data;
                              Strand      : in Strand_Type;
                              Seed_Size   : in Integer;
                              Mask_Size   : in Integer;
                              Mask        : in Integer;
                              Seeds       : in out Seed_Segments) is
    Root_Size : constant Integer := Seed_Size - Mask_Size;
    Seq   : Nucleotidic_Sequence renames Sequence.Nucleotides;
    type Adr_Array is array(Integer range <>) of Integer;
    M_Adr : Adr_Array(0..Mask_Size) := (others => 0);
    Adr : Adr_Array(0..Root_Size) := (others => 0);
    Code_Length : constant Integer := (Nb_Symbol ** Root_Size)-1;
    Code_Size : constant Integer := Root_Size + 1;
    M_Code_Length : constant Integer := (Nb_Symbol ** Mask_Size)-1;
    M_Code_Size : constant Integer := Mask_Size + 1;
    M_Shift : constant Integer := Nb_Symbol ** (Mask_Size-1);
    Shift : constant Integer := Nb_Symbol ** (Root_Size-1);
    Current_First,Current_Last,Code : Integer;
    M_Current_First,M_Current_Last,M_Code : Integer;
    Segment : Segment_Type;
    From,To,Limit,Increment : Integer;
    I    : Integer;

  begin
    -- Pour tous les segments generes, l'identificateur sera le meme
    -- pour cette sequence
    Segment.Id := Sequence.Id;
    Segment.Strand := Strand;
    -- En fonction du brin considere, l'analyse se fait dans un sens ou 
    -- dans l'autre
    if Strand = Watson then
      From := Sequence.Start;
      To   := Sequence.Stop;
      Increment := 1;
    else
      From := Sequence.Stop;
      To   := Sequence.Start;
      Increment := -1;
    end if;
    -- Boucle sur la zone a etudier
    loop
      begin
        -- Initialisation
        Current_First := 0;
        Current_Last  := 0;
        Code  := 0;
        M_Current_First := 0;
        M_Current_Last  := 1;
        M_Code  := 0;
        -- Construire le codage initial de selection
        M_Adr(0) := 0; -- cas du premier intervalle
        I := From - Increment;
        Limit := From + Increment * (Mask_Size-2);
        while I /= Limit loop
          if I = To then return; end if;
          I := I + Increment;
          M_Adr(M_Current_Last) := Tnu(Strand,Seq(I));
          M_Code := M_Code * Nb_Symbol + M_Adr(M_Current_Last);
          M_Current_Last := M_Current_Last + 1;
        end loop;
        -- Construire le codage initial du premier segment
        Limit := From + Increment * (Seed_Size-2);
        while I /= Limit loop
          if I = To then return; end if;
          I := I + Increment;
          Adr(Current_Last) := Tnu(Strand,Seq(I));
          Code := Code * Nb_Symbol + Adr(Current_Last);
          Current_Last := Current_Last + 1;
        end loop;
        -- parcours de l'arbre
        loop
          if I = To then return; end if;
          I := I + Increment;
          -- determiner le code de selection
          M_Adr(M_Current_Last) := Adr(Current_First);
          M_Code := ((M_Code - (M_Adr(M_Current_First)*M_Shift)) * Nb_Symbol) +
            M_Adr(M_Current_Last);
          -- calcul des indices pour le code
          M_Current_First := (M_Current_First + 1) mod M_Code_Size;
          M_Current_Last := (M_Current_Last + 1) mod M_Code_Size;
          -- determiner le code
          Adr(Current_Last) := Tnu(Strand,Seq(I));
          Code := ((Code - (Adr(Current_First)*Shift)) * Nb_Symbol) +
            Adr(Current_Last);
          -- calcul des indices pour le code
          Current_First := (Current_First + 1) mod Code_Size;
          Current_Last := (Current_Last + 1) mod Code_Size;
          -- enregistrer l'intervalle
          if M_Code = Mask then
            if Strand = Watson then
              Segment.Start := I - Seed_Size + 1;
              Segment.Stop  := I;
            else
              Segment.Start := I;
              Segment.Stop  := I + Seed_Size - 1;
            end if;
            Insert(Seeds(Code),Segment);
          end if;         
        end loop;
      exception
        when UNDEFINED_NUCLEOTIDE =>
          From := I + Increment;
      end;
      -- on quitte si From = to
      exit when From = To;
    end loop;
  end Get_Seeds_By_Mask;

end Assirc_Seeds;
