--------------------------------------------------------------------------
-- Cpu_Times.Basic                          Copyrigth ENS ULM 1998-1998 --
-- Pierre VINCENS                   V 01.000                 03/04/1998 --
--------------------------------------------------------------------------
-- VERSION Unix pour SUN Solaris 2.4/Linux
--------------------------------------------------------------------------
with System;
with Interfaces.C; use type Interfaces.C.Long;

with Ada.Text_Io; use Ada.Text_Io;
with Ada.Integer_Text_Io; use Ada.Integer_Text_Io;
with Ada.Calendar; use Ada.Calendar;

with System_Constants; use System_Constants;

package body Cpu_Times.Basic is

  subtype Clock_T is Interfaces.C.Long;
  
  package Duration_IO is new Ada.Text_Io.Fixed_Io(Duration);
  use Duration_IO;

  
--------------------------------------------------------------------------
-- Prendre en compte l'origine du lancement du process                  --
--------------------------------------------------------------------------
-- On utilise pour cela les fonctionnalits du paquetage Ada.calendar
-- C'est donc portable. Attention cependant, l'origine est dfinie au
-- moment de l'laboration de ce paquetage
--------------------------------------------------------------------------
  Starting_Time : constant Time := Clock;

--------------------------------------------------------------------------
-- Interface avec la fonction times du systme                          --
--------------------------------------------------------------------------
-- Cette fonction retourne le temps CPU consomm par le process depuis
-- qu'il a dbut.
--------------------------------------------------------------------------
  type Tms_Type is record
    User_Time          : Clock_T;
    System_Time        : Clock_T;
    Child_User_Time    : Clock_T;
    Child_System_Time  : Clock_T;
  end record;

  function Times(Buffer : in System.Address) return Clock_T;
  pragma Import(C,Times,"times");

--------------------------------------------------------------------------
-- Affichage de la date                                                 --
--------------------------------------------------------------------------
  function Date_To_String(Date : in Time) return String is
    Text : String(1..10);
  begin
    Put(Text(1..2),Day(Date));
    if Text(1) = ' ' then Text(1) := '0'; end if;
    Text(3) := '/';
    Put(Text(4..5),Month(Date));
    if Text(4) = ' ' then Text(4) := '0'; end if;
    Text(6) := '/';
    Put(Text(7..10),Year(Date));
    for I in 7..9 loop
      if Text(I) = ' ' then Text(I) := '0'; end if;
    end loop;
    return Text;
  end Date_To_String;

  function Date_To_String return String is
  begin
    return Date_To_String(Clock);
  end Date_To_String;

  function Time_To_String(Date : in Time) return String is
    Text    : String(1..8);
    Hours   : constant Integer  := Integer(Seconds(Date))/3600;
    Minuts  : constant Integer  := 
      (Integer(Seconds(Date))-Hours*3600)/60;
    Seconds : constant Integer := 
      Integer(abs(Ada.Calendar.Seconds(Date)-Duration(Hours*3600+Minuts*60)));
  begin
    Put(Text(1..2),Hours);
    if Text(1) = ' ' then Text(1) := '0'; end if;
    Text(3) := ':';
    Put(Text(4..5),Minuts);
    if Text(4) = ' ' then Text(4) := '0'; end if;
    Text(6) := ':';
    Put(Text(7..8),Seconds);
    if Text(7) = ' ' then Text(7) := '0'; end if;
    return Text;
  end Time_To_String;

  function Time_To_String return String is
  begin
    return Time_To_String(Clock);
  end Time_To_String;
  
  function Time_To_String(Times : in Duration) return String is
    Hours   : constant Integer  := Integer(Times)/3600;
    Minuts  : constant Integer  := (Integer(Times)-Hours*3600)/60;
    Seconds : constant Duration := abs(Times-Duration(Hours*3600+Minuts*60));
    Hours_Text : constant String := Integer'Image(Hours);
    Text : String(1..Hours_Text'Length+7);
    Current : Natural := Text'First+Hours_Text'Length-1;
  begin
    Text(1..Current) := Hours_Text(Hours_Text'First+1..Hours_Text'Last) & ":";
    Put(Text(Current+1..Current+2),Minuts);
    if Text(Current+1) = ' ' then Text(Current+1) := '0'; end if;
    Text(Current+3) := ':';
    Put(Text(Current+4..Text'Last),Seconds,1,0);
    if Text(Current+4) = ' ' then Text(Current+4) := '0'; end if;
    return Text;
  end Time_To_String;
--------------------------------------------------------------------------
-- Procedure accessible  l'utilisateur qui donne les diffrents temps  --
--------------------------------------------------------------------------
  procedure Print_Times is
    Tms     : Tms_Type;
    Cpu     : constant Clock_T := Times(Tms'Address);
    Elapsed : constant Duration := Clock - Starting_Time;
    Hours   : constant Integer  := Integer(Elapsed)/3600;
    Minutes : constant Integer  := (Integer(Elapsed)-Hours*3600)/60;
    Seconds : constant Duration := 
      abs(Elapsed - Duration(Hours*3600+Minutes*60));
  begin
    Put("Elapsed time: ");
    Put(Duration(Tms.User_Time)/Clk_Tck,0,2); 
    Put("u ");
    Put(Duration(Tms.System_Time)/Clk_Tck,0,2); 
    Put("s ");
    Put(Duration(Tms.User_Time+Tms.System_Time)/Clk_Tck,0,2); 
    Put("t ");
    Put(Duration(Tms.Child_User_Time)/Clk_Tck,0,2); 
    Put("cu ");
    Put(Duration(Tms.Child_System_Time)/Clk_Tck,0,2); 
    Put("cs ");
    Put(Elapsed,0,2); 
    Put("r ");
    Put(Hours,0);
    Put(':');
    Put(Minutes,0);
    Put(':');
    Put(Seconds,0,2);
    New_Line;
  end Print_Times;

  function Elapsed_Cpu_Time return Duration is
    Tms     : Tms_Type;
    Cpu     : constant Clock_T := Times(Tms'Address);
  begin
    return Duration(Tms.User_Time+Tms.System_Time+
		    Tms.Child_User_Time+Tms.Child_System_Time)/Clk_Tck;
  end Elapsed_Cpu_Time;

  function Elapsed_Real_Time return Duration is
  begin
    return Clock - Starting_Time;
  end Elapsed_Real_Time;

  function Elapsed_Times return Cpu_Duration is
    Tms     : Tms_Type;
    Cpu     : constant Clock_T := Times(Tms'Address);
    Elapsed : Cpu_Duration := 
      (Real_Time   => Clock - Starting_Time,
       User_Time   => Duration(Tms.User_Time)/Clk_Tck,
       System_Time => Duration(Tms.System_Time)/Clk_Tck);
  begin
    return Elapsed;
  end Elapsed_Times;
  
end Cpu_Times.Basic;
