--------------------------------------------------------------------------
-- Dassirc_Alignment                        Copyrigth ENS ULM 1996-1999 --
-- Pierre VINCENS                   V 00.000                 04/06/1999 --
--------------------------------------------------------------------------
with Ada.Unchecked_Deallocation;
with Ada.Text_Io; use Ada.Text_Io;
with Type_Specifications.IO; use Type_Specifications.IO;
with Strlib; use Strlib;
with Strlib.Io; use Strlib.Io;
with Options_Util; use Options_Util;
  
with Sequence_Specifications; use Sequence_Specifications;
with Sequence_Specifications.Basic; use Sequence_Specifications.Basic;

with Sequence_Alignment; use Sequence_Alignment;
with Sequence_Alignment.Basic; use Sequence_Alignment.Basic;
with Sequence_Alignment.Print; use Sequence_Alignment.Print;
with Sequence_Alignment.Global; use Sequence_Alignment.Global;
with Sequence_Alignment.Local; use Sequence_Alignment.Local;

with Assirc_Specifications; use Assirc_Specifications;
with Assirc_Specifications.Database; use Assirc_Specifications.Database;
with Assirc_Global_Variables; use Assirc_Global_Variables;
with Assirc_Database_Functions; use Assirc_Database_Functions;

package body Dassirc_Alignment is

--------------------------------------------------------------------------
-- Acces aux bibliothques                                              --
--------------------------------------------------------------------------
  use Type_Specifications.IO.Int_Io;
  use Type_Specifications.IO.Real_Io;
  use Type_Specifications.Real_Lib;


  procedure Perform_One_Alignment
    (Sequences            : in Assirc_Sequence_Array_Access;
     Pair                 : in out Segment_Pair;
     Summary              : out Alignment_Summary_Model;
     Method               : in Alignment_Method;
     Max_Length           : in Integer := 3500;
     Adjust               : in Integer := 0;
     Match_Weight         : in Integer := 100;
     Mismatch_Weight      : in Integer := -90;
     Gap_Weight           : in Integer := -500;
     Gap_Length_Weight    : in Integer := -30;
     With_End_Weight      : in Boolean := False;
     Similarity_Threshold : in Integer := -1;
     Randomization        : in Integer := 0;
     Preserve             : in Integer := 1) is
    T_Seq1,T_Seq2  : Sequence_Data;
    Alignments     : Alignments_Access;
    L_Start,L_Stop : Integer;
  begin
    -- Calcul sur le brin watson. On ajuste aprs.
    L_Start := Integer'Max(Pair.Seg1.Start - Adjust,
			   Sequences(Pair.Seg1.Id).Nucleotides'First);
    L_Stop  := Integer'Min(Pair.Seg1.Stop + Adjust,
			   Sequences(Pair.Seg1.Id).Nucleotides'Last);
    -- Dtermination de la position corrige de dbut et de fin
    -- Initialisation de la premire squence
    case Pair.Seg1.Strand is
      when Watson =>
	T_Seq1.Name := Sequences(Pair.Seg1.Id).Name;
	T_Seq1.Identifier := 
	  Sequences(Pair.Seg1.Id).Identifier; -- & Image(Pair.Seg1.Strand);
	T_Seq1.Model := Sequences(Pair.Seg1.Id).Model;
	T_Seq1.Format := Sequences(Pair.Seg1.Id).Format;
	T_Seq1.Length := Sequences(Pair.Seg1.Id).Length;
	T_Seq1.Nucleotides := Sequences(Pair.Seg1.Id).Nucleotides;
	T_Seq1.Start := L_Start;
	T_Seq1.Stop := L_Stop;
      when Crick =>
	T_Seq1.Name := Sequences(Pair.Seg1.Id).Name;
	T_Seq1.Identifier := 
	  Sequences(Pair.Seg1.Id).Identifier;-- & Image(Pair.Seg1.Strand);
	T_Seq1.Model := Sequences(Pair.Seg1.Id).Model;
	T_Seq1.Format := Sequences(Pair.Seg1.Id).Format;
	T_Seq1.Length := Sequences(Pair.Seg1.Id).Length;
	T_Seq1.Start := Sequences(Pair.Seg1.Id).Length-L_Stop+1;
	T_Seq1.Stop := Sequences(Pair.Seg1.Id).Length-L_Start+1;
	T_Seq1.Nucleotides :=  
	  new Nucleotidic_Array(L_Start..L_Stop);
	T_Seq1.Nucleotides.all := 
	  Sequences(Pair.Seg1.Id).Nucleotides(L_Start..L_Stop);
	Reverse_Sequence(T_Seq1);
	Cmp_Sequence(T_Seq1);
      when Both =>
	raise INVALID_STRAND;
    end case;
    -- Dtermination de la position corrige de dbut et de fin
    -- Calcul sur le brin watson. On ajuste aprs.
    L_Start := Integer'Max(Pair.Seg2.Start - Adjust,
			   Sequences(Pair.Seg2.Id).Nucleotides'First);
    L_Stop  := Integer'Min(Pair.Seg2.Stop + Adjust,
			   Sequences(Pair.Seg2.Id).Nucleotides'Last);
    -- Initialisation de la seconde squence
    case Pair.Seg2.Strand is
      when Watson =>
	T_Seq2.Name := Sequences(Pair.Seg2.Id).Name; 
	T_Seq2.Identifier := 
	  Sequences(Pair.Seg2.Id).Identifier;-- & Image(Pair.Seg2.Strand);
	T_Seq2.Model := Sequences(Pair.Seg2.Id).Model;
	T_Seq2.Format := Sequences(Pair.Seg2.Id).Format;
	T_Seq2.Length := Sequences(Pair.Seg2.Id).Length;
	T_Seq2.Nucleotides := Sequences(Pair.Seg2.Id).Nucleotides;
	T_Seq2.Start := L_Start;
	T_Seq2.Stop := L_Stop;
      when Crick =>
	T_Seq2.Name := Sequences(Pair.Seg2.Id).Name;
	T_Seq2.Identifier := 
	  Sequences(Pair.Seg2.Id).Identifier;-- & Image(Pair.Seg2.Strand);
	T_Seq2.Model := Sequences(Pair.Seg2.Id).Model;
	T_Seq2.Format := Sequences(Pair.Seg2.Id).Format;
	T_Seq2.Length := Sequences(Pair.Seg2.Id).Length;
	T_Seq2.Start := Sequences(Pair.Seg2.Id).Length-L_Stop+1;
	T_Seq2.Stop := Sequences(Pair.Seg2.Id).Length-L_Start+1;
	T_Seq2.Nucleotides :=  
	  new Nucleotidic_Array(L_Start..L_Stop);
	T_Seq2.Nucleotides.all := 
	  Sequences(Pair.Seg2.Id).Nucleotides(L_Start..L_Stop);
	Reverse_Sequence(T_Seq2);
	Cmp_Sequence(T_Seq2);
      when Both =>
	raise INVALID_STRAND;
    end case;
    if ((T_Seq1.Stop - T_Seq1.Start + 1 > Max_Length) or
	(T_Seq2.Stop - T_Seq2.Start + 1 > Max_Length)) then
      Put_The_N_Last_Characters(Summary.Name1,T_Seq1.Identifier);
      Summary.Start1 := Pair.Seg1.Start;
      Summary.Stop1 := Pair.Seg1.Stop;
      Put_The_N_Last_Characters(Summary.Name2,T_Seq2.Identifier);
      Summary.Start2 := Pair.Seg2.Start;
      Summary.Stop2 := Pair.Seg2.Stop;
    else
      case Method is
	when Global_Alignment =>
	  Perform_Global_Alignment(T_Seq1,T_Seq2,Alignments, 
				   T_Seq1.Start,T_Seq1.Stop,
				   T_Seq2.Start,T_Seq2.Stop,
				   "",
				   Match_Weight,Mismatch_Weight,
				   Gap_Weight,Gap_Length_Weight,
				   DVH,One,No,With_End_Weight,
				   Similarity_Threshold,
				   Randomization,Preserve);
	  
	when Local_Alignment =>
	  Perform_Local_Alignment(T_Seq1,T_Seq2,Alignments,
				  T_Seq1.Start,T_Seq1.Stop,
				  T_Seq2.Start,T_Seq2.Stop,
				  "",
				  Match_Weight,Mismatch_Weight,
				  Gap_Weight,Gap_Length_Weight,
				  DVH,One,No,Similarity_Threshold,
				  Randomization,Preserve);
	when others =>
	  raise NOT_IMPLEMENTED;
      end case;
      -- Modifier les positions de dbut et de fin de l'alignement pour
      -- tenir compte du brin sur lequel s'est fait le calcul et
      -- imprimer le rsultat
      -- ATTENTION, on ne ne considre que le premier rsultat
      -- il ne doit y en avoir qu'un seul vu les parametres
      -- de l'alignement
      for I in Alignments'First..Alignments'First loop
	Alignments(I).Strand1 := Pair.Seg1.Strand;
	case Pair.Seg1.Strand is
	  when Watson =>
	    null;
	  when Crick =>
	    Alignments(I).Start1 := 
	      Sequences(Pair.Seg1.Id).Length - Alignments(I).Start1 + 1;
	    Alignments(I).Stop1  := 
	      Sequences(Pair.Seg1.Id).Length - Alignments(I).Stop1 + 1;
	  when others =>
	    raise INVALID_STRAND;
	end case;
	Alignments(I).Strand2 := Pair.Seg2.Strand;
	case Pair.Seg2.Strand is
	  when Watson =>
	    null;
	  when Crick =>
	    Alignments(I).Start2 := 
	      Sequences(Pair.Seg2.Id).Length - Alignments(I).Start2 + 1;
	    Alignments(I).Stop2  := 
	      Sequences(Pair.Seg2.Id).Length - Alignments(I).Stop2 + 1;
	  when others =>
	    raise INVALID_STRAND;
	end case;
      end loop;
      -- Retourner le rsultat
      Summary := To_Alignment_Summary(Alignments(Alignments'First).all);
    end if;
    -- Librer la place rserve pour dterminer la squence rverse
    -- dans le cas d'un brin Crick
    if Pair.Seg1.Strand = Crick then
      Free(T_Seq1.Nucleotides);
    end if;
    if Pair.Seg2.Strand = Crick then
      Free(T_Seq2.Nucleotides);
    end if;
  end Perform_One_Alignment;

end Dassirc_Alignment;
  
