--------------------------------------------------------------------------
-- Dassirc_Builder_Options                  Copyrigth ENS ULM 1996-1998 --
-- Pierre VINCENS                   V 01.000                 01/04/1998 --
--------------------------------------------------------------------------
with Ada.Text_Io; use Ada.Text_Io;
with Ada.Command_Line; use Ada.Command_Line;
with Options_Util; use Options_Util;

package body Dassirc_Builder_Options is
  
  procedure Get_Options(Options : in out Builder_Options) is
      Count : Positive := 1;
  begin
    if Argument_Count = 0 then
      raise ARGUMENT_ERROR;
    end if;
    while Count <= Argument_Count loop
      if Is_An_Alias(Argument(Count),"-ALL",2) then
        -- Support de toutes les actions, le calcul des graines tant combin
        Options.Supported_Actions := All_Actions;
	Options.Computing_Method := Simple;
      elsif Is_An_Alias(Argument(Count),"-LOCAL_COMBINED",2) then
        -- Support de la gnration de graines en mode local combin
        Options.Supported_Actions(Seed_Computing) := True;
      	Options.Supported_Actions(Expand_Computing) := False;
      	Options.Computing_Method := Local_Combined;
      elsif Is_An_Alias(Argument(Count),"-COMBINED",2) then
        -- Support de la gnration de graines en mode combin
        Options.Supported_Actions(Seed_Computing) := True;
      	Options.Supported_Actions(Expand_Computing) := False;
      	Options.Computing_Method := Combined;
      elsif Is_An_Alias(Argument(Count),"-SIMPLE",2) then
        -- Support de la gnration de graines en mode simple
        Options.Supported_Actions(Seed_Computing) := True;
      	Options.Computing_Method := Simple;
      elsif Is_An_Alias(Argument(Count),"-EXPAND",3) then
        -- Support du calcul des extensions
        Options.Supported_Actions(Expand_Computing) := True;
      elsif Is_An_Alias(Argument(Count),"-ETASK",3) then
        -- Nombre de taches  activer pour le calcul des extensions
	Count := Count + 1;
	begin
	  Options.Expand_Task := Opt_Int(Argument(Count));
	  if Options.Expand_Task < 1 then
	    Options.Expand_Task := 2;
	  end if;
	exception
	  when Constraint_Error | Data_Error =>
	    Options.Expand_Task := 2;
	end;
      elsif Is_An_Alias(Argument(Count),"--BOOT_SERVER",13) then
        -- Cette option est utilise par Glade en version distribue
        -- pour prciser le numro du port de la socket
        -- On l'ignore ainsi que la valeur de la socket donne en argument suivant
        Count := Count + 1;
      elsif Is_An_Alias(Argument(Count),"--SLAVE",7) then
        -- Cette option est utilise par Glade en version distribue
        -- pour indiquer qu'il s'agit d'un process esclave. On l'ignore.
        null;
      else
        -- raise ARGUMENT_ERROR;
	Put_Line("Invalid options value: " & Argument(Count));
      end if;
      Count := Count + 1;
    end loop;

    if Options.Supported_Actions = No_Action then
      -- Si aucune action supporte, on active par dfaut le support
      -- de toutes les actions, le calcul des graines tant combin
        Options.Supported_Actions := All_Actions;
	Options.Computing_Method := Simple;
    end if;
  end Get_Options;

  procedure Print_Options is
  begin
    Put_Line("DASSIRC_BUILDER: argument error");
    Put_Line("    syntax is: dassirc_builder  ");
    Put_Line("                      [-All]");
    Put_Line("                      [-Local_Combined]");
    Put_Line("                      [-Combined]");
    Put_Line("                      [-Simple]");
    Put_Line("                      [-EXpand]");
    Put_Line("                      [-ETask <value>]");
    New_Line;
  end Print_Options;

end Dassirc_Builder_Options;
