--------------------------------------------------------------------------
-- Dassirc_Channel                          Copyrigth ENS ULM 1996-1998 --
-- Pierre VINCENS                   V 01.000                 23/03/1998 --
--------------------------------------------------------------------------
-- Ce paquetage prend en charge le transfer des squences entre le
-- dispatcher et les clients.
--------------------------------------------------------------------------
with Sequence_Specifications; use Sequence_Specifications;
with Sequence_Specifications.Basic; use Sequence_Specifications.Basic;

with Ada.Unchecked_Conversion;
with Ada.Streams; use Ada.Streams;
with Ada.Text_Io; use Ada.Text_Io;
with Ada.Exceptions;

package body Dassirc_Channel is

  package Integer_Io is new Ada.Text_Io.Integer_Io(Integer);
  use Integer_Io;

  function To_Stream_Element is new 
    Ada.Unchecked_Conversion(Nucleotide_Type,Stream_Element);

  function To_Nucleotide is new 
    Ada.Unchecked_Conversion(Stream_Element,Nucleotide_Type);

  procedure Open_Input_Connection(Accepting_Socket : in out Socket_FD;
				  Port             : in out Positive) is
    Max_Port : Integer := 32700;
  begin
    Socket(Accepting_Socket,AF_INET, SOCK_STREAM);
    -- v1.0.2 Accepting_Socket := Socket(AF_INET, SOCK_STREAM);
    Setsockopt(Accepting_Socket,SOL_SOCKET,SO_REUSEADDR,1);
    loop
      begin
	Bind(Accepting_Socket,Port);
	exit;
      exception
	when Constraint_Error =>
	  Put_Line("Socket unbinded. Check another port");
	  if Port > Max_Port then
	    raise;
	  end if;
	  Port := Port + 1;
      end;
    end loop;
    Listen(Accepting_Socket);
  end Open_Input_Connection;

  procedure Receive_Sequence(Sequence : in Nucleotidic_Sequence;
			     Accepting_Socket : in out Socket_FD) is
    Incomming_Socket : Socket_FD; 
    Current : Integer := Sequence'First;
  begin
    Put_Line("Receiving...");
    loop
      begin
	Accept_Socket(Accepting_Socket,Incomming_Socket);
	exit;
      exception
	when Constraint_Error => 
	  Put_Line("Retry Accept_Socket...");
      end;
    end loop;
    -- v1.0.2 Incomming_Socket := Accept_Socket(Accepting_Socket);
    Put_Line("Accepting sockets");
    while Current <= Sequence'Last loop
      declare
	Data : constant Stream_Element_Array := Receive(Incomming_Socket);
      begin
	--Put_Line("Received buffer");
	--Put("Stream_Length       : "); Put(S_Data'Length); New_Line;
	--Put("Stream_Array_Size   : "); Put(S_Data'Size); New_Line;
	--Put("Stream_Size         : "); Put(Stream_Element'Size); New_Line;
	--Put("Character_Size      : "); Put(Character'Size); New_Line;
	Put(".");
	for I in Data'Range loop
	  Sequence(Current) := To_Nucleotide(Data(I));
	  Current := Current + 1;
	  exit when Current > Sequence'Last;
	end loop;
      end;
    end loop;
    New_Line;
    Put_Line("Disconnecting.");
    Shutdown(Incomming_Socket,Send);
    Put_Line("Disconnected.");
  end Receive_Sequence;

  procedure Close_Input_Connection(Accepting_Socket : in out Socket_FD) is
  begin
    Shutdown(Accepting_Socket,Receive);
  end Close_Input_Connection;

  procedure Send_Sequence(Sequence : in Nucleotidic_Sequence;
			  From,To  : in Natural;
			  Host     : in String;
			  Port     : in Positive) is
    Buffer_Size : constant Stream_Element_Offset := 3000;
    Buffer : Stream_Element_Array(1..Buffer_Size);
    Outgoing_Socket : Socket_FD;
    Current : Integer := From;
  begin
    Put_Line("Opening socket");
    -- v1.0.2 Outgoing_Socket := Socket(AF_INET, SOCK_STREAM);
    begin
      Socket(Outgoing_Socket, AF_INET, SOCK_STREAM);
    exception
      when Constraint_Error =>
	raise Unavailable_Socket;
    end;
    Put_Line("Connecting");
    Connect(Outgoing_Socket,Host,Port);
    Send_Loop: while Current <= To loop
      for I in Buffer'Range loop
	Buffer(I) := To_Stream_Element(Sequence(Current));
	Current := Current + 1;
	if Current > To then
	  --Put_Line("Sending last buffer");
	  Send(Outgoing_Socket,Buffer(Buffer'First..I));
	  exit Send_Loop;
	end if;
      end loop;
      --Put_Line("Sending buffer");
      Send(Outgoing_Socket,Buffer);
    end loop Send_Loop;
    Put_Line("Disconnecting");
    Shutdown(Outgoing_Socket,Send);
    Put_Line("Disconnected");
  end Send_Sequence;
  

end Dassirc_Channel;
