--------------------------------------------------------------------------
-- Dassirc_Database                         Copyrigth ENS ULM 1996-1998 --
-- Pierre VINCENS                   V 00.000                 10/04/1996 --
-- Pierre VINCENS                   V 01.000                 26/03/1998 --
--------------------------------------------------------------------------
-- Gestion distribue de la base de donnes des fragments similaires 
-- Cette partition gre un arbre contenant l'ensemble des paires trouves
-- NOTES:
--   1/ La base est maintenue interne  ce paquetage. Toutes les oprations
--      sur la base doivent donc se faire via cette interface. La gestion
--      se fait sous la forme d'un arbre unique.
--   2/ Ce paquetage est destin  tre utilis exclusivement par la 
--      version distribue. Pour la version non distribue, il faut utiliser
--      le paquetage quivalenet Assirc_Database.
--   3/ Ce paquetage propose deux mchanismes d'accs  la base de donnes
--      synchrone ou asynchrone. Le premier pose le problme de bloquer
--      l'appelant durant les oprations de test d'inclusion o d'ajout
--      d'une paire. Le second conduit  tester davantage de paires
--      car plusieurs paires appartenant  la mme rgion peuvent
--      tre values avant que cette rgion ne soit effectivement
--      enregistre dans la base.  
--------------------------------------------------------------------------
with Type_Specifications; use Type_Specifications;

with Assirc_Specifications; use Assirc_Specifications;

with Dassirc_Common; use Dassirc_Common;

package Dassirc_Database is

  pragma Remote_Call_Interface;

--------------------------------------------------------------------------
-- Constante d'interaction avec la base                                 --
--------------------------------------------------------------------------
  Max_Pairs_For_Registering : constant := 200;
  -- Le nombre de paires transmis avec la base. Sauf si le modele de
  -- calcul 4 est utilis, o il doit imprativement y avoir cohrence
  -- entre la taille des paquets envoyes

  Max_Pairs_For_Diffusion : constant := Max_Pairs_For_Registering;
  -- Cette valeur doit etre obligatoirement plus grande ou egale
  -- (efficacite maximale) au nombre de paires mis.
  
--------------------------------------------------------------------------
-- Initialisation et terminaison                                        --
--------------------------------------------------------------------------
  procedure Start(Excluded_Marge     : in Integer;
		  Intersection_Marge : in Integer);
  -- Activation de la base de donnes et enregistrement des paramtres 
  -- de gestion
  
  procedure Stop;
  -- Indiquer  la base de donnes que tout est termin. Cela a pour effet
  -- de dtruire la tache de mise a jour du dispatcher.

  procedure Update;
  -- Cette procedure sert  mettre la jour la liste des process utilise
  -- par la gestion de la base de donnes. Elle doit etre appele aprs
  -- l'initialisation de chaque client

--------------------------------------------------------------------------
-- Test de l'inclusion de graines dans la base de donnes               --
--------------------------------------------------------------------------
-- Le but de ces sous-programmes est de vrifier le fait qu'une graine
-- se trouve dj dans une rgion connue comme similaire ou non
-- Il y a deux stratgies possibles:
--   1/ La fonction Included prend deux segments, vrifie le critre
--      et retourne la rponse. La procedure Are_Included prend un
--      tableau de paires, les testent et retournent exclusivement
--      celles pour lesquelles il faut raliser l'inclusion. Dans ces
--      deux cas, il s'agit d'un mchanisme synchrone.
--   2/ La procdure To_Check_Inclusion rcupre les paires  tester
--      les mmorisent et rend la main  l'appelant. La base de donnes
--      ralise les tests en arrire plan et les mmorisent dans une 
--      structure FIFO. Les procdures Next_Pair_To_Expand et 
--      Next_Pairs_To_Expand retournent respectivement une ou plusieurs
--      paires  tester. Avant de dbuter l'envoi de graines  tester
--      suivant cette mthode, il faut informer la base  l'aide de la
--      procedure To_Check_Inclusion_Start. Cela a pour effet d'initialiser
--      la tache alloue au test interne des inclusions et de rserver
--      l'espace ncessaire  la pile FIFO. Lorsque toutes les graines 
--      devant tre testes ont t transmises  la base de donnes, 
--      la base doit tre informe par un appel de la procdure 
--      To_Check_Inclusion_Stop. Aprs cet appel, il reste possible de
--      tester des inclusions avec la fonction Included (mode synchrone) 
--      et d'ajouter des paires dans la base. Rien n'interdit d'activer
--      et de dsactiver ce mchanisme plusieurs fois. La difficult
--      pourra alors venir du contrle de la terminaison des clients.
--      La fonction No_Pair_To_Expand retourne le vrai si la pile FIFO
--      est vide et si le processus d'envoi de paires  tester est
--      invalid. 
--------------------------------------------------------------------------   
  NO_PAIR_AVAILABLE : exception;
  -- Indique qu'aucune paire n'est disponible dans la pile des
  -- graines  tendre. Attention, cette exception n'implique
  -- pas qu'il n'y aura pas de paires ultrieurement disponible.
  -- La rponse  cette question est donnee par la fonction
  -- No_Pair_To_Expand.

  function Included(Seg1,Seg2 : in Segment_Type) return Boolean;
  -- Test de la prsence d'une graine dans l'arbre.

  function Are_Included(Pairs : in Segment_Pairs) return Answers_Array;
  -- Test de la prsence de graines dans l'arbre. Le tableau de boolens
  -- retourn indique pour chaque paire si elle est incluse ou pas.
  -- Le range de answers est identique  celui de Pairs.
  
  procedure To_Check_Inclusion_Start;
  procedure To_Check_Inclusion_Stop;
  -- Initialisation et terminaison du gestionnaire de test des inclusions
  -- d'une manire synchrone

  function No_Pair_To_Expand return Boolean;
  -- Retourne vrai si la pile FIFO interne contenant les paires  tester
  -- est vide ET si le processus d'envoi de paires  tester est invalid.    

  procedure To_Check_Inclusion(Pairs         : in Segment_Pairs;
			       Wait_For_Next : out Boolean);
  -- Test de la prsence de graines dans l'arbre. En cas de saturation
  -- de la pile interne, cet appel est temporairement bloquant. Le
  -- paramtre Wait_For_Next permet d'indiquer  l'appelant que la
  -- pile a depasse le seuil de remplissage normal.

  procedure Next_Pair_To_Expand(Pair : out Segment_Pair);
  -- Renvoie une paire  tester. En cas de pile interne vide,
  -- l'exception NO_PAIR_AVAILABLE est leve

  procedure Next_Pairs_To_Expand(Pairs : out Segment_Pairs;
				 Last  : out Integer);
  -- Renvoie Pairs'Length paires  tester. Au cas o la pile ne contient 
  -- pas suffisament de paires  tendre, un nombre plus restreint de
  -- paire est retourn, l'indice de la dernire tant Last. Une
  -- valeur de last plus petite que Pairs'First indique qu'il n'y
  -- avait pas de graines  tester. Cela n'implique pas qu'il n'y en
  -- aura pas ultrieurement. Pour le savoir il faut utiliser la
  -- fonction No_Pair_To_Expand.

  procedure Wait_End_Of_Process;
  -- Cette procdure attend que l'acceptation des tests d'inclusions
  -- aient t ferme, que toutes les demandes d'inclusions aient t 
  -- testes et que toutes les demandes d'expansion aient t prises
  -- en compte. Par contre, elle n'attend pas l'enregistrement de
  -- toutes les paires dans l'arbre.
      

--------------------------------------------------------------------------
-- Diffusion des paires dans les base de donnes locales                --
--------------------------------------------------------------------------
  procedure Add_Client(Client : in Client_Id);
  -- Inscription d'un client au service de mise  jour d'une base de donnes
  -- locale. Lors de l'inscription, toutes les paires dj enregistres
  -- dans la base sont prpar pour etre diffus vers le client. Attention
  -- cela gnre une charge pour le serveur et donc il est prfrable
  -- d'incrire les clients utilisant ce service ds le dbut de la recherche

  procedure Remove_Client(Client : in Client_Id);
  -- Desinscription d'un client au service. Cette opration dclenche
  -- la suppression des paires en attente d'etre transmise vers ce client

  procedure Last_Pairs_From_Client(Client : in View_Id;
				   Pairs  : out Segment_Pairs; 
				   Last   : out Integer;
				   Next   : out Boolean);
  -- Obtention par le client des paires de la base de donnes centralise
  -- pour mise  jour de la base locale du client. Tant que last est
  -- diffrent de 0, il y a des paires  diffuser vers le client.
  -- Le tableau Pairs doit avoir au moins une taille gale  
  -- Max_Pairs_For_Diffusion. S'il est plus grand, la place restante
  -- ne sera pas utilis. La valeur de next indique s'il y a d'autres
  -- paquets en attente pour ce client.
  

--------------------------------------------------------------------------
-- Ajout de paires dans la base de donnes                              --
--------------------------------------------------------------------------
-- L'ajout peut se faire une paire  la fois ou par paquet
-- Pour des raisons d'efficacite, les paires sont enregistrees dans
-- une pile intermediaire, une tache specifique et independante
-- prend en charge l'ajout dans l'arbre. Cela evite de bloquer le
-- client pendant une periode trop longue
--------------------------------------------------------------------------
  --procedure Append_Pair_In_Tree(Pair : in Segment_Pair);
  -- Ajout d'une paire dans l'arbre

  procedure Append_Pairs_In_Tree(Pairs : in Segment_Pairs);
  -- Ajout de plusieurs paires dans l'arbre

  procedure Append_Pairs_In_Tree(Client : in View_Id;
				 Pairs  : in Segment_Pairs);
  -- Ajout de plusieurs paires dans l'arbre avec identification du
  -- client emmeteur. Le but est d'viter de lui retransmettre ces
  -- paires et de reduire ainsi le traffic rseau
  
  procedure Registering_Is_Achieved;
  -- Permet de signifier a la base de donner qu'il n'y aura plus de 
  -- nouveaux paquets de paires envoyes
  
  procedure Wait_End_Of_Registering;
  -- Permet d'attendre la fin de l'enregistrement des paires dans la base
  
--------------------------------------------------------------------------
-- Parcours linaire de la base de donnes                              --
--------------------------------------------------------------------------
-- Build_Entries et Release_Entries initialisent et closent respectivement
-- cette fonctionnalit. Les paires peuvent tre obtenues successivement
-- par Next_Pair ou Next_Pairs suivant si l'on en veut une ou plusieurs 
-- la fois
--------------------------------------------------------------------------
 
  procedure Build_Entries;
  -- Cration d'un tableau contenant un pointeur vers toutes les entres
  -- de la base

  procedure Release_Entries;
  -- Destruction du tableau prcedent

  procedure Next_Pair(Pair : out Segment_Pair);
  -- Obtention de la paire suivante dans le tableau

  procedure Next_Pairs(Minimal_Length : in Integer;
		       Pairs          : out Segment_Pairs;
		       Last           : out Integer);
  -- Obtention de Pairs'Length paires vrifiant la condition d'tre de 
  -- taille suprieure  Minimal_Length. Last donne l'indice dans le tableau 
  -- Pairs de la dernire paire mmorise. L'exception OUT_OF_ENTRIES est
  -- leve s'il n'y a plus de paires disponibles 

end Dassirc_Database;
