--------------------------------------------------------------------------
-- Dassirc_Dispatcher                       Copyrigth ENS ULM 1996-1998 --
-- Pierre VINCENS                   V 01.000                 06/03/1998 --
-- Pierre VINCENS                   V 01.001                 29/11/2000 --
--------------------------------------------------------------------------
-- Paquetage RCI auquel se connecte les clients pour obtenir leur job
-- et a l'issue de celui-ci retourner le resultat
--------------------------------------------------------------------------
-- V 01.001
-- Le type protg dispatcher est reporte dans un paquetage spar
-- pour rsoudre un problme de dpendance croise
--------------------------------------------------------------------------
with Type_Specifications; use Type_Specifications;

with Sequence_Specifications; use Sequence_Specifications;

with Sequence_Alignment; use Sequence_Alignment;

with Assirc_Specifications; use Assirc_Specifications;

with Dassirc_Common; use Dassirc_Common;

package Dassirc_Dispatcher is

  pragma Remote_Call_Interface;

  procedure Start;
  -- L'appel de cette procedure est fait par le manager qui est
  -- dans la mme partition lorsqu'il a analys les parametres
  -- de la ligne de commande et lu et prpar les squences.
  -- Tant que cette procdure n'a pas t active, les autres
  -- fonctions du paquetage sont bloquantes. 

  procedure Stop;
  -- L'appel de cette procdure indique la terminaison du dispatcher.
  -- A partir de ce moment, les appels  state lve une erreur 
  -- DISPATCHER_COMPLETED

  procedure Wait_The_Last_Client;
  -- Cette procedure est bloquante et permet d'attendre la deconnexion
  -- du dernier client

  procedure Client_Registering(Client           : out Client_Id;
                               Supported_Action : in out Action_List;
                               Computing_Method : in Method_Of_Seed_Computing);
  -- L'appel de cette procedure se fait lors de l'initialisation d'un client
  -- et informe le dispatcher de sa disponibilit. Action dfinit les
  -- actions supportes par le client. Les actions peuvent tre forces
  -- par le serveur

  procedure Client_Disconnecting(Client : in Client_Id);
  -- Cette fonction permet de dclarer la dconnexion d'un client
  -- au serveur
  
  procedure Client_Aborting(Client : in Client_Id);
  -- Cette fonction permet de declarer au serveur qu'un client se deconnecte
  -- sans avoir conclu sa tache
  
  function Get_Index_Of_Last_Sequences return Accession_Type;
  -- Cette fonction retourne l'indice de la derniere sequence. On assumera
  -- que l'index de la premiere sequence est 1
  
  procedure Get_Sequences_Info(Index   : in Accession_Type;
                               Id      : out Accession_Type;
                               Strand  : out Strand_Type;
                               From    : out Integer;
                               To      : out Integer;
			       Length  : out Integer);
  -- Cette fonction retourne le parametres de la sequence (identificateur,
  -- brin, origine et terminaison) en position index.

  function Get_Sequences_Identifier(Index   : in Accession_Type)
				    return String;
  -- Cette fonction retourne l'identicateur de la squence en position
  -- Index

  procedure Get_Sequence(Client  : in Accession_Type; 
                         From,To : in Natural;
                         Host    : in String;
                         Port    : in Positive);
  -- Cette procdure permet au client de rcuprer la squence. 
  -- Compte-tenu des performances sous-jacentes
  -- aux transferts par rseau, la squence doit tre transfere
  -- via une socket specifique

  function Get_Parameters return Searching_Parameters;
  -- L'appel de cette fonction permet au client de recopier
  -- les paramtres de la recherche et de l'alignement dans son 
  -- espace mmoire propre. ATTENTION: le shift doit tre obtenu 
  -- indpendamment par la fonction Get_Searching_Shift.

  function Get_Searching_Shift return Shift_Type;
  -- L'appel de cette fonction permet au client de recuprer
  -- les valeurs du shift

  procedure Get_Region(Client    : in Client_Id;
                       Zone1     : out Segment_Type;
		       Zone2     : out Segment_Type;
		       Z1_Xonly  : out Boolean;
		       Z2_Xonly  : out Boolean);
  procedure Region_Completed(Client : in Client_Id);
  -- Le client appelle Get_Region pour obtenir du serveur les zones
  -- qu'il doit traiter. Le client prend en charge cette region
  -- localement et enregistre en fin de cycle les rsultats dans 
  -- la base de donnes globale. Aprs terminaison de cette tache,
  -- il informe le dispatcher de son achvement.
  -- Une distribution par rgion est incompatible avec les autres
  -- mthodes.

  function Get_Seed_Mask(Client : in Client_Id) return Integer;
  procedure Mask_Completed(Client : in Client_Id; Mask : in Integer);
  -- Le client appelle Get_Seed_Mask pour obtenir du serveur
  -- un masque pour la recherche des graines. Le client doit 
  -- alors gnrer les graines correspondantes et tester 
  -- chacune des rgions candidates par extension. Chaque
  -- rsultat doit tre enregistree dans la base de donnes.
  -- Lorsqu'il a termine, il informe le serveur en appelant
  -- Mask_Completed.

  procedure Expand_Computing_Start(Client : in Client_Id);
  procedure Expand_Computing_Stop(Client : in Client_Id);
  -- Le client informe qu'il prend en charge ou qu'il arrete une charge
  -- d'extension de graines

  procedure Alignment_Computing_Start(Client : in Client_Id);
  procedure Alignment_Computing_Stop(Client : in Client_Id);
  -- Le client informe qu'il prend en charge ou qu'il arrete une charge
  -- d'alignement des paires pour validation
  
  procedure Wait_End_Of_Search;
  -- Cette procedure est bloquante et attend la fin de la recherche
  -- Elle doit etre appele par le manager
  
  procedure Print_Database(Name           : in String;
                           Minimal_Length : in Integer);
  -- Cette procdure est appele par le manager. Elle n'est
  -- activable que lorsque la recherche des graines avec tous les 
  -- masques possibles a t ralise

  procedure Start_Alignment;
  -- Cette procdure est appele par le manager. Elle n'est
  -- activable que lorsque la recherche des graines avec tous les 
  -- masques possibles a t ralise. Elle dclenche l'initialisation
  -- de la phase de validation par alignement
  procedure Stop_Alignment;
  -- Cette procedure est appele par le manager aprs que toutes
  -- les paires aient t testes et que l'impression aient t faite

  procedure Append_Summaries(Summaries : in Alignment_Summaries_Array);
  -- Cette procedure enregistre les rsultats des alignements dans 
  -- une liste. Elle est appele par les clients builder quand 
  -- l'alignement a t initialis par le manager

  procedure Print_Valid_Pairs(Name : in String);
  -- Cette procdure est appele par le manager pour dclencher
  -- l'impression des paires valides
  
  function Terminated return Boolean;
  -- Le moniteur peut savoir si le programme est en phase de terminaison

end Dassirc_Dispatcher;
