--------------------------------------------------------------------------
-- Dassirc_Dispatcher_Management            Copyrigth ENS ULM 1996-2000 --
-- Pierre VINCENS                   V 01.000                 29/11/2000 --
--------------------------------------------------------------------------
-- V 01.000: Ce paquetage est cr pour rsoudre un problme de
-- dpendance croise entre dassirc_dispatcher et dassirc_database
-- Le type protg Dispatcher est desormais report dans ce paquetage.
--------------------------------------------------------------------------
with Ada.Text_Io; use Ada.Text_Io;

with Fifo,Linked_List;

with Type_Specifications; use Type_Specifications;

with Sequence_Alignment; use Sequence_Alignment;

with Dassirc_Common; use Dassirc_Common;

with Assirc_Specifications; use Assirc_Specifications;

package Dassirc_Dispatcher_Management is

--------------------------------------------------------------------------
-- Constantes                                                           --
--------------------------------------------------------------------------
  Max_Putative_Pairs : constant Integer := 100;
  -- Donne la taille du buffer contenant les paires  tester
  
--------------------------------------------------------------------------
-- Gestion de la liste contenant les paires valides                    --
--------------------------------------------------------------------------
  package Alignment_Summaries_List is 
    new Linked_List(Alignment_Summary_Model);
  use Alignment_Summaries_List;
  
--------------------------------------------------------------------------
-- Procdures de reprises des exceptions (pb des types proteges)        --
--------------------------------------------------------------------------
  type Exception_Id is 
    (Null_Exception,Not_Connected_Exception,Invalid_Status_Exception,
     Invalid_Mask_Exception,No_Seed_Available_Exception,
     Too_Many_Clients_Exception);
  procedure Reraise_Exception(Id : in Exception_Id);
  
--------------------------------------------------------------------------
--  Types pour mmoriser les jobs en cours lors de l'abort d'un client  --
--------------------------------------------------------------------------
  type Job_Of_Segment_Pairs is record
    Zone1     : Segment_Type;
    Zone2     : Segment_Type;
    Z1_Xonly  : Boolean;
    Z2_Xonly  : Boolean;
  end record;
    
  type Job_Aborted_Type is record 
    Action   : Action_Type;              -- Type d'action
    Method   : Method_Of_Seed_Computing; -- Mthode utilise
    Mask     : Integer := -1;            -- Valeur du masque concerne
    Pairs    : Job_Of_Segment_Pairs;     -- Rgion concerne
    --From,To  : Integer := -1;            -- Rgion concerne
  end record;
  
  package Job_Aborted_Fifo is new Fifo(Job_Aborted_Type);
  use Job_Aborted_Fifo;
  
--------------------------------------------------------------------------
-- Types pour mmoriser l'tat des clients                              --
--------------------------------------------------------------------------
  type Dispatcher_State is (Waiting,Dispatching,Completed,
			    Aligning,Validated,Terminated);
    
  type Client_Status is record
    Connect  : Boolean := False;         -- Etat du client
    Accepted : Action_List := No_Action; -- Action acceptable par le client
    Running  : Action_List := No_Action; -- Action en cours sur le client
    Method   : Method_Of_Seed_Computing; -- Mthode utilise par ce client
    Mask     : Integer := -1;            -- Valeur du masque affect
    Pairs    : Job_Of_Segment_Pairs;     -- Rgion a calculer
    --From,To  : Integer := -1;            -- Rgion calcule
  end record;
  type Clients_Array is array(Client_Id) of Client_Status;
  
--------------------------------------------------------------------------
--  Types pour maintenir la position des rgions  affecter             --
--------------------------------------------------------------------------
    
  type Position_Type is record
    Id : Accession_Type := Accession_Type'First;
    From, To : Integer;
  end record;
  
--------------------------------------------------------------------------
-- Gestion du dispatcher  l'aide d'un type protg                     --
--------------------------------------------------------------------------
  
  protected Dispatcher is
    -- waiting     : non initialis
    -- dispatching : prt  distribuer des taches
    -- completed   : toutes les taches de recherche ont t compltes
    -- aligning    : pret  aligner
    -- validated   : toutes les taches d'alignement ont termines
    -- terminated  : tout est fini
    -- aborted     : client avort
    entry Start;
    -- pour indiquer que les donnes sont disponibles.
    entry Stop;
    -- pour indiquer qu'il faut arreter
    entry Ready;
    -- pour mettre en attente les requetes necessitant que les
    -- donnees soient disponibles
    entry New_Client(Client : out Client_Id;
                     Action : in out Action_List;
                     Method : in Method_Of_Seed_Computing;
		     Error  : out Exception_Id); 
    -- pour connecter un nouveau client (retourne son numro)
    -- l'action peut tre force par le serveur
    procedure Abort_Client(Client : in Client_Id;
			   Error  : out Exception_Id);
    -- pour dconnecter un client qui abandonne sa tache. Cette procedure
    -- est appelable  tout moment sans controle prealable de l'etat
    -- du client. Elle doit donc etre employee avec prudence.
    entry Close_Client(Client : in Client_Id;
		       Error  : out Exception_Id);
    -- pour dconnecter un client normalement
    entry Wait_The_Last;
    -- pour attendre que le dernier client soit deconnecte
    entry Assign_Region(Client    : in Client_Id;
                        Zone1     : out Segment_Type;
                        Zone2     : out Segment_Type;
                        Z1_Xonly  : out Boolean;
                        Z2_Xonly  : out Boolean;
			Error     : out Exception_Id);
    -- pour affecter une nouvelle rgion
    entry Terminate_Region(Client : in Client_Id;
			   Error  : out Exception_Id);
    -- por dclarer qu'une rgion a t compltement prise en compte
    entry Assign_Mask(Client : in Client_Id; 
		      Mask   : out Integer;
		      Error  : out Exception_Id);
    -- pour affecter un nouveau masque pour la graine
    entry Terminate_Mask(Client : in Client_Id; 
			 Mask   : in Integer;
			 Error  : out Exception_Id);
    -- pour dclarer que toutes les graines pour un masque ont t calcules
    entry Assign_Expand_Computing(Client : in Client_Id;
				  Error  : out Exception_Id);
    -- pour affecter a un client le traitement des extensions
    entry Terminate_Expand_Computing(Client : in Client_Id;
				     Error  : out Exception_Id);
    -- pour qu'un client puisse informer qu'il a termin cette tache
    entry Assign_Alignment_Computing(Client : in Client_Id;
				     Error  : out Exception_Id);
    -- pour qu'un client puisse informer qu'il a activ l'alignment     
    entry Terminate_Alignment_Computing(Client : in Client_Id;
					Error  : out Exception_Id);
    -- pour qu'un client puisse informer qu'il a termin cette tache
    entry Ready_To_Align;
    -- pour indiquer que toutes les requetes de gnration des paires
    -- potentielles ont ete completees et que l'on peut passer  la
    -- phase de validation 
    entry Activate_Alignment;
    -- pour faire ce qu'il faut pour dclencher la phase de validation par
    -- alignement.
    -- soit positionner le seuil minimal de longueur o l'on tente 
    -- l'alignement
    entry Terminate_Alignment;
    -- pour liberer la place associe aux paires valides
    entry Append_Summaries(Summaries : in Alignment_Summaries_Array);
    -- pour ajouter les donnes  la liste lie
    entry Print_Summaries(File : in Ada.Text_Io.File_Type);
    -- pour imprimer les paires valides  lorsque l'alignement est fini
    procedure View_State;
    -- Afficher l'etat du programme pour debug
    function Program_Terminated return Boolean;
    -- pour savoir si le programme est en phase de terminaison
  private
    State : Dispatcher_State := Waiting;
    -- Etat du programme
    Clients : Clients_Array;
    -- tableau mmorisant l'tat des clients
    Last_Client : Client_Id := Client_Id'First;
    -- Identificateur du dernier client actif
    Clients_Connected : Boolean := True;
    -- Indique s'il reste un client connecte
    Jobs_Aborted : Job_Aborted_Fifo.Stack_Handler;
    -- La pile FIFO des eventuels jobs a reaffecter
    Position_1,Position_2 : Position_Type;
    -- position du dbut de la prochaine rgion  analyser
    P1_Xonly,P2_Xonly : Boolean := False;
    -- Indique s'il faut considrer la recherche d'un segment contre lui
    -- meme. Au depart les deux valeurs sont a Vrai, puis des que le 1er
    -- segment a ete traite une fois, on passe P1_Xonly a Faux, puis
    -- des que ce mme segment a ete compar avec toutes les paires
    -- on passe P2_Xonly a faux.
    Computerized_Pairs_Of_Regions : Natural := 0;
    -- Nombre de couples de regions traites
    Current_Mask : Integer := 0; 
    -- valeur du prochain masque  dispatcher
    Last_Mask : Integer; 
    -- valeur du dernier masque  dispatcher
    Expand_Computing_Required : Boolean := False;
    -- le dispatching d'expansion des graines est activ
    Minimal_Region : Integer := 500;
    -- la taille minimale qu'il doit restee pour constituer une region
    -- en dessous de cette valeur, on agrandit la region precedente
    -- de maniere a aller jusqu' la fin de la sequence.
    Summaries_List : Alignment_Summaries_List.List_Handler;
    -- la liste ou l'on enregistre tous les alignements valids
  end Dispatcher;

end Dassirc_Dispatcher_Management;
