--------------------------------------------------------------------------
-- Dassirc_Statistics_Io                    Copyrigth ENS ULM 1996-1999 --
-- Pierre VINCENS                   V 01.000                 07/06/1999 --
--------------------------------------------------------------------------
-- Affichage de l'avance des process
--------------------------------------------------------------------------
with Ada.Text_Io; use Ada.Text_Io;
with Ada.Calendar; use Ada.Calendar;

with Type_Specifications; use Type_Specifications;
with Type_Specifications.IO; use Type_Specifications.IO;
with Cpu_Times; use Cpu_Times;
with Cpu_Times.Basic; use Cpu_Times.Basic;

with Dassirc_Common; use Dassirc_Common;
with Dassirc_Dispatcher; use Dassirc_Dispatcher;

package body Dassirc_Statistics_IO is

--------------------------------------------------------------------------
-- Acces aux bibliothques                                              --
--------------------------------------------------------------------------
  package View_Io is new Ada.Text_Io.Integer_Io(View_Id);
  package Duration_IO is new Ada.Text_Io.Fixed_Io(Duration);
  use Duration_IO,Int_Io,View_Io;
  

--------------------------------------------------------------------------
-- Impression                                                           --
--------------------------------------------------------------------------
  procedure Print_Status(Status : in Status_Type) is
  begin
    Put(' ');
    case Status is
      when Unknown    => Put('U');
      when Waiting    => Put('W');
      when Running    => Put('R');
      when Terminated => Put('T');
      when Aborted    => Put('A');
    end case;
    Put(' ');
  end Print_Status;

  procedure Display(State : in Partition_State_Array) is
    Sum_Utime : Duration := 
      State(Dispatcher_Id).Times.User_Time + State(Database_Id).Times.User_Time;
    Sum_Stime : Duration := 
      State(Dispatcher_Id).Times.System_Time + 
      State(Database_Id).Times.System_Time;
    Max_Rtime : Duration := 
      Duration'Max(State(Dispatcher_Id).Times.Real_Time,
		   State(Database_Id).Times.Real_Time);
    Sum_Constituted, Sum_Tested, Sum_Registered, Sum_Validated : Integer := 0;
  begin
    Put("Monitoring at: ");
    Put(Date_To_String); Put("  "); Put(Time_To_String);
    Put(" Elapsed: "); 
    Put(Time_To_String(State(Dispatcher_Id).Times.Real_Time));
    New_Line;
    Put_Line("Dispatcher:"); 
    Put(Dispatcher_Id,3);
    Print_Status(State(Dispatcher_Id).Status);
    Put(State(Dispatcher_Id).Times.User_Time,6,1); Put("u ");
    Put(State(Dispatcher_Id).Times.System_Time,6,1); Put("s ");
    Put(State(Dispatcher_Id).Times.Real_Time,6,1); Put("r ");
    New_Line;
    Put_Line("Database:");
    Put(Database_Id,3);
    Print_Status(State(Database_Id).Status);
    Put(State(Database_Id).Times.User_Time,6,1); Put("u ");
    Put(State(Database_Id).Times.System_Time,6,1); Put("s ");
    Put(State(Database_Id).Times.Real_Time,6,1); Put("r ");
    Put(State(Database_Id).Database_Count,10); Put("R");
    New_Line;
    Put_Line("Clients:");
    for I in Client_Id loop
      if State(I).Status /= Unknown then
	-- On affiche l'tat
	Put(Integer(I),3);
	Print_Status(State(I).Status);
	Put(State(I).Times.User_Time,6,1); Put("u ");
	Put(State(I).Times.System_Time,6,1); Put("s ");
	Put(State(I).Times.Real_Time,6,1); Put("r ");
	Put(State(I).Constituted_Count,10); Put("C ");
	Put(State(I).Tested_Count,10); Put("T ");
	Put(State(I).Registered_Count,7); Put("R ");
	Put(State(I).Validated_Count,7); Put("V");
	New_Line;
	-- On somme pour obtenir le cumul
	Sum_Constituted := Sum_Constituted + State(I).Constituted_Count;
	Sum_Tested := Sum_Tested + State(I).Tested_Count;
	Sum_Registered := Sum_Registered + State(I).Registered_Count;
	Sum_Validated := Sum_Validated +State(I).Validated_Count;
	Sum_Utime := Sum_Utime + State(I).Times.User_Time;
      	Sum_Stime := Sum_Stime + State(I).Times.System_Time;
	-- et on prend le temps rel maximal
	Max_Rtime := Duration'Max(Max_Rtime,State(I).Times.Real_Time); 
      end if;
    end loop;
    -- Afficher les totaux
    Put("Sum   ");
    Put(Sum_Utime,6,1); Put("u ");
    Put(Sum_Stime,6,1); Put("s ");
    Put(Max_Rtime,6,1); Put("r ");
    Put(Sum_Constituted,10); Put("C ");
    Put(Sum_Tested,10); Put("T ");
    Put(Sum_Registered,7); Put("R ");
    Put(Sum_Validated,7); Put("V");
    New_Line;
  end Display;

end Dassirc_Statistics_Io;
