--------------------------------------------------------------------------
-- Dassirc_Utils                            Copyrigth ENS ULM 1996-1998 --
-- Pierre VINCENS                   V 01.000                 16/03/1998 --
--------------------------------------------------------------------------
with Dassirc_Channel; use Dassirc_Channel;
with Ada.Text_Io; use Ada.Text_Io;
with Ada.Calendar; use Ada.Calendar;
with Ada.Exceptions; use Ada.Exceptions;

with Sockets; use Sockets;
with Sockets.Naming; use Sockets.Naming;
with Strlib; use Strlib;
with Cpu_Times.Counters; use Cpu_Times.Counters;

package body Dassirc_Utils is

  package Duration_Io is new Ada.Text_Io.Fixed_Io(Duration);
  use Duration_Io;
  package Integer_Io is new Ada.Text_Io.Integer_Io(Integer);
  use Integer_Io;

  function Load_Sequences return Assirc_Sequence_Array_Access is

    Accepting_Socket : Socket_FD;
    Port : Positive := 7587;
    Host : constant String := Host_Name;
    Zero : Time := Clock;
    From,To,Length : Integer;
    Strand : Strand_Type;
    Last : Accession_Type;
    Sequences : Assirc_Sequence_Array_Access;

    task Read_Sequences is
      entry Get(Index : in Accession_Type);
      entry Finish;
    end Read_sequences;

    task body Read_Sequences is
      Id : Accession_Type;
    begin
      Cpu_Counters.Add("Read_Sequences");
      Waiting: loop
	select
	  accept Get(Index : in Accession_Type) do
	    Id := Index;
	  end Get;
	  Receive_Sequence(Sequences(Id).Nucleotides,Accepting_Socket);
	or
	  accept Finish;
	  exit Waiting;
	end select;
      end loop Waiting;
      Cpu_Counters.Remove;
    exception
      when The_Error : others =>
	Put_Line("Exception raised in Read_Sequences");
	Put_Line(Ada.Exceptions.Exception_Information(The_Error));
    end Read_Sequences;

  begin
    -- Obtenir le nombre de sequences a traiter
    Last := Get_Index_Of_Last_Sequences;
    -- Creer l'espace necessaire a l'enregistrement des sequences
    Sequences := new Assirc_Sequence_Array(Accession_Type(1)..Last); 
    -- Ouvrir la socket 
    Put_Line("Initializing on host: " & Host & ".");
    Open_Input_Connection(Accepting_Socket,Port);
    -- Recuperer chaque squence  traiter
    for I in Sequences'Range loop
      -- d'abord l'index
      Get_Sequences_Info(I,Sequences(I).Id,Strand,From,To,Length);
      Sequences(I).Nucleotides := new Nucleotidic_Array(From..To);
      Sequences(I).Model  := DNA;
      Sequences(I).Strand := Strand;
      Sequences(I).Start  := From;
      Sequences(I).Stop   := To;
      Sequences(I).Length := Length;
      Copy(Get_Sequences_Identifier(I),Sequences(I).Identifier);
      -- les autres champs ne sont pas initialises car non utilise
      -- au niveau du client
      --
      -- puis la sequence elle-meme
      -- on se prpare a la recevoir
      Put_Line("Waiting sequence" & Accession_Type'Image(I));
      Read_Sequences.Get(I);
      -- et on en demande l'envoi
      Put_Line("Requesting sequence" & Accession_Type'Image(I));
      Get_Sequence(Sequences(I).Id,From,To,Host,Port);
    end loop;
    -- Clore la tache
    Read_Sequences.Finish;
    -- Clore la socket
    Put_Line("Closing connection.");
    Close_Input_Connection(Accepting_Socket);
    Put_Line("Finish...");
    Put("Elapsed_time: ");
    Put(Clock - Zero,6,3);
    New_Line;
    return Sequences;
  exception
    when Unavailable_Socket =>
      Put_Line("Socket_unavailable: free memory");
      for I in Sequences'Range loop
	Clear(Sequences(I));
      end loop;
      Free(Sequences);
      Put_Line("                    kill task");
      abort Read_Sequences;
      raise Sequences_Not_Loaded;
  end Load_Sequences;

  procedure Print_Sequence(Seq : in Nucleotidic_Sequence;
			   Name : in String) is
    Fic : File_Type;
    Buffer : constant Natural := 60;
    From : Natural := Seq'First;
    To   : Natural;
  begin
    Create(Fic,Out_File,Name);
    Put_Line(Fic,"> " & Name);
    while From <= Seq'Last loop
      To := From + Buffer - 1;
      if To > Seq'Last then
	To := Seq'Last;
      end if;
      for I in From..To loop
	Put(Fic,To_Character(Seq(i)));
      end loop;
      New_Line(Fic);
      From := To + 1;
    end loop;
    Close(Fic);
  end Print_Sequence;

end Dassirc_Utils;
