--------------------------------------------------------------------------
-- Linked_List                              Copyrigth ENS ULM 1986-1999 --
-- Pierre VINCENS                   V 01.000                 22/10/1993 --
-- Pierre VINCENS                   V 01.001                 25/05/1998 --
-- Pierre VINCENS                   V 01.002                 30/06/1999 --
--------------------------------------------------------------------------
with Ada.Unchecked_Deallocation;

package body Linked_List is

procedure Free is new Ada.Unchecked_Deallocation(List_Node,List_Ptr);
procedure Free_List is
  new Ada.Unchecked_Deallocation(List_Entries_Array,List_Entries);

procedure Free(Entries : in out List_Entries) is
begin
  Free_List(Entries);
end Free;

procedure Append(List    : in out List_Handler;
                 Element : in Element_Type;
                 Address : in List_Ptr := Nil) is
  Adr,Ptr : List_Ptr;
begin
  if Address = null then
    Adr := List.Last;
  else
    Adr := Address;
  end if;
  Ptr := new List_Node'(null,null,Element);
  List.Count := List.Count + 1;
  if Adr = null then
    -- la liste est vide
    List.First := Ptr;
    List.Last  := Ptr;
  else
    if Adr.Next = null then
      -- l'element est a ajouter en fin de liste
      Adr.Next := Ptr;
      Ptr.Previous := Adr;
      List.Last := Ptr;
    else
      -- l'element est a ajouter dans le milieu de la liste
      Ptr.Previous := Adr;
      Ptr.Next := Adr.Next;
      Adr.Next := Ptr;
      Ptr.Next.Previous := Ptr;
    end if;
  end if;
end Append;

procedure Insert(List    : in out List_Handler;
                 Element : in Element_Type;
                 Address : in List_Ptr := Nil) is
  Adr,Ptr : List_Ptr;
begin
  if Address = null then
    Adr := List.First;
  else
    Adr := Address;
  end if;
  Ptr := new List_Node'(null,null,Element);
  List.Count := List.Count + 1;
  if Adr = null then
    -- la liste est vide
    List.First := Ptr;
    List.Last  := Ptr;
  else
    if Adr.Previous = null then
      -- l'element est a inserer en debut de liste
      Adr.Previous := Ptr;
      Ptr.Next := Adr;
      List.First := Ptr;
    else
      -- l'element est a inserer dans le milieu de la liste
      Ptr.Previous := Adr.Previous;
      Ptr.Next := Adr;
      Adr.Previous := Ptr;
      Ptr.Previous.Next := Ptr;
    end if;
  end if;
end Insert;

function First(List    : in List_Handler) return Element_Type is
  -- retourne le premier element de la liste
begin
  return List.First.Element;
end First;

function Last(List    : in List_Handler) return Element_Type is
  -- retourne le dernier element de la liste
begin
  return List.Last.Element;
end Last;

function First(List    : in List_Handler) return List_Ptr is
  -- retourne l'adresse du dernier element de la liste
begin
  return List.First;
end First;

function Last(List    : in List_Handler) return List_Ptr is
  -- retourne l'adresse du dernier element de la liste
begin
  return List.Last;
end Last;

function Consult(Address : in List_Ptr) return Element_Type is
begin
  return Address.Element;
end Consult;

procedure Delete(List    : in out List_Handler;
                 Address : in out List_Ptr) is
begin
  if Address = List.First then
    -- l'element a supprimer est celui qui est en tete de la liste
    List.First := Address.Next;
    if Address.Next = null then
      -- La liste ne contenait qu'un seul lment, on met  jour le pointeur
      -- de fin de liste
      List.Last := null;
    else
      -- La liste contient plusieurs lment, on met  jour le pointeur
      -- du second lment
      Address.Next.Previous := null;
    end if;
  elsif Address = List.Last then
    -- l'element a supprimer est celui qui est en queue de la liste
    List.Last := Address.Previous;
    if Address.Previous /= null then
      -- Ce test doit toujours etre vrai car sinon Address = List.First
      -- est vrai
      Address.Previous.Next := null;
    end if;
  else
    -- l'element est dans le milieu de la liste
    Address.Previous.Next := Address.Next;
    Address.Next.Previous := Address.Previous;
  end if;
  -- On supprime reellement l'element
  List.Count := List.Count - 1;
  Free(Address);
end Delete;

procedure Modify(Address : in List_Ptr;
                 Data    : in out Data_Command) is
begin
  Command(Address.Element,Data);
end Modify;

procedure Assign(Address : in List_Ptr;
                 Data    : in Data_Command) is
begin
  Command(Address.Element,Data);
end Assign;

procedure Replace(Address : in List_Ptr;
                  Element : in Element_Type) is
begin
  Address.Element := Element;
end Replace;

function Find_First_Element(List : in List_Handler) return List_Ptr is
  Ptr : List_Ptr := List.First;
begin
  -- parcourir la liste
  while Ptr /= null loop
    if Feature(Ptr.Element) then
      return Ptr;
    end if;
    Ptr := Ptr.Next;
  end loop;
  return null;
end Find_First_Element;

function Find_Element(List : in List_Handler;
                      Data : in Data_Type) return List_Ptr is
  Ptr : List_Ptr := List.First;
begin
  -- parcourir la liste
  while Ptr /= null loop
    if Feature(Ptr.Element,Data) then
      return Ptr;
    end if;
    Ptr := Ptr.Next;
  end loop;
  return null;
end Find_Element;

procedure Apply(List : in List_Handler) is
  Ptr,Next : List_Ptr := List.First;
begin
  -- parcourir la liste
  while Ptr /= null loop
    Next := Ptr.Next;
    Command(Ptr.Element);
    Ptr := Next;
  end loop;
end Apply;

procedure Apply_With_Data(List : in List_Handler;
                          Data : in out Data_Command) is
  Ptr,Next : List_Ptr := List.First;
begin
  -- parcourir la liste
  while Ptr /= null loop
    Next := Ptr.Next;
    Command(Ptr.Element,Data);
    Ptr := Next;
  end loop;
end Apply_With_Data;

procedure Apply_With_Address(List : in List_Handler) is
  Ptr,Next : List_Ptr := List.First;
begin
  -- parcourir la liste
  while Ptr /= null loop
    Next := Ptr.Next;
    Command(Ptr.Element,Ptr);
    Ptr := Next;
  end loop;
end Apply_With_Address;

procedure Apply_If(List      : in List_Handler;
                   Condition : in Data_Feature;
                   Data      : in out Data_Command) is
  Ptr,Next : List_Ptr := List.First;
begin
  -- parcourir la liste
  while Ptr /= null loop
    Next := Ptr.Next;
    if Feature(Ptr.Element,Condition) then
      Command(Ptr.Element,Data);
    end if;
    Ptr := Next;
  end loop;
end Apply_If;

procedure Apply_While(List      : in List_Handler;
                      Condition : in Data_Feature;
                      Data      : in out Data_Command) is
  Ptr,Next : List_Ptr := List.First;
begin
  -- parcourir la liste
  while Ptr /= null loop
    Next := Ptr.Next;
    exit when not Feature(Ptr.Element,Condition);
    Command(Ptr.Element,Data);
    Ptr := Next;
  end loop;
end Apply_While;

procedure Apply_Until(List      : in List_Handler;
                      Data      : in out Data_Command) is
  Ptr,Next : List_Ptr := List.First;
begin
  -- parcourir la liste
  while Ptr /= null loop
    Next := Ptr.Next;
    Command(Ptr.Element,Data);
    exit when Feature(Ptr.Element,Data);
    Ptr := Next;
  end loop;
end Apply_Until;

procedure Apply_If_To_First(List      : in List_Handler;
                            Condition : in Data_Feature;
                            Data      : in out Data_Command;
                            Apply     : out Boolean) is
  Ptr,Next : List_Ptr := List.First;
begin
  -- parcourir la liste
  while Ptr /= null loop
    Next := Ptr.Next;
    if Feature(Ptr.Element,Condition) then
      Command(Ptr.Element,Data);
      Apply := True;
      return;
    end if;
    Ptr := Next;
  end loop;
  Apply := False;
end Apply_If_To_First;

procedure Apply_And_Remove_When(List : in out List_Handler;
                                Data : in out Data_Type) is
  Ptr,Next : List_Ptr := List.First;
  Remove   : Boolean;
begin
  -- parcourir la liste
  while Ptr /= null loop
    Next := Ptr.Next;
    Command(Ptr.Element,Data,Remove);
    if Remove then
      Delete(List,Ptr);
    end if;
    Ptr := Next;
  end loop;
end Apply_And_Remove_When;

procedure Reset(List : in out List_Handler) is
  Ptr,Next : List_Ptr := List.First;
begin
  -- liberer la place occupee par la liste
  while Ptr /= null loop
    Next := Ptr.Next;
    Free(Ptr);
    Ptr := Next;
  end loop;
  -- mettre a jour la tete et la queue de la liste
  List.First := null;
  List.Last  := null;
  List.Count := 0;
end Reset;

procedure Clean_And_Reset(List : in out List_Handler) is
  Ptr,Next : List_Ptr := List.First;
begin
  -- liberer la place occupee par la liste
  while Ptr /= null loop
    Next := Ptr.Next;
    Clean(Ptr.Element);
    Free(Ptr);
    Ptr := Next;
  end loop;
  -- mettre a jour la tete et la queue de la liste
  List.First := null;
  List.Last  := null;
  List.Count := 0;
end Clean_And_Reset;

function Is_Empty(List : in List_Handler) return Boolean is
begin
  return List.First = null;
end Is_Empty;

function Count(List : in List_Handler) return Natural is
begin
  return List.Count;
end Count;

procedure Get_Entries(List : in List_Handler; Entries : in out List_Entries) is
  Ptr : List_Ptr := List.First;
  I   : Natural;
begin
  -- initier un tableau de la taille necessaire
  Entries := new List_Entries_Array(1..List.Count);
  I := Entries'First;
  -- parcourir la liste
  while Ptr /= null loop
    Entries(I) := Ptr;
    I := I + 1;
    Ptr := Ptr.Next;
  end loop;
end Get_Entries;

end Linked_List;

