--------------------------------------------------------------------------
-- Options_Util                             Copyrigth ENS ULM 1986-1999 --
-- Pierre VINCENS                   V 01.000                 26/07/1996 --
-- Pierre VINCENS                   V 01.001                 04/03/1999 --
--------------------------------------------------------------------------
with Ada.Text_Io; use  Ada.Text_Io;
with Type_Specifications.IO; use Type_Specifications.IO;
with Ada.Command_Line; use  Ada.Command_Line;

package body Options_Util is

  use Int_Io;
  use Real_Io;

  function Upcase(Value : in String) return String is
    -- Cette fonction transforme les caracteres minuscules
    -- en caracteres majuscules.
    Chaine : String(Value'Range) := Value;
  begin
    for I in Chaine'Range loop
      if Chaine(I) >= Character'Val(97) and
        Chaine(I) <= Character'Val(122) then
        Chaine(I) := Character'Val(Character'Pos(Chaine(I)) - 32);
      end if;
    end loop;
    return Chaine;
  end Upcase;

  function Is_An_Alias(Value : in String;
                       Name : in String;
                       Min  : in Natural := 2) return Boolean is
    -- cette fonction retourne vrai si la chaine value s'inscrit dans
    -- la chaine name. La chaine value doit comporter au moins les min
    -- premiers caracteres. On ne tient pas compte des lettres
    -- minuscules ou majuscules.
    Src    : constant String := Upcase(Value);
    Target : constant String := Upcase(Name);
  begin
    -- verifier la validite de value
    if Value'Length < Min or Value'Length > Name'Length then
      return False;
    end if;
    -- verifier la validite des caracteres donnes
    if Src = Target(Target'First..Target'First+Src'Length-1) then
      return True;
    end if;
    return False;
  end Is_An_Alias;

  function Root_Name(Src: in String) return String is
    Pos,Debut,Fin : Integer;
  begin
    if Src'Length = 0 then
      return "";
    end if;
    Debut := Src'First;
    Pos := Src'Last;
    Fin := Pos;
    while Pos >= Src'First loop
      case Src(Pos) is
        when '/' => Debut := Pos + 1; exit;
        when '.' => Fin := Pos - 1;
        when others => null;
      end case;
      Pos := Pos - 1;
    end loop;
    return Src(Debut..Fin);
  end Root_Name;

  function Root_Name(Src: in Varstring) return String is
  begin
    if Length(Src) = 0 then
      return "";
    end if;
    return Root_Name(Src.Value.all);
  end Root_Name;

  function Opt_Int(Str : in String) return Integer is
    Last,Value : Integer;
  begin
    Get(Str,Value,Last);
    return Value;
  end Opt_Int;

  function Opt_Real(Str : in String) return Real is
    Last : Integer;
    Value : Real;
  begin
    Get(Str,Value,Last);
    return Value;
  end Opt_Real;

  function Opt_Sequence_Format(Str : in String) return Sequence_Format is
  begin
    if Is_An_Alias(Str,"ASCII",1) then
      return Plain_Ascii;
    elsif Is_An_Alias(Str,"DNA_STRIDER",5) then
      return Dna_Strider;
    elsif Is_An_Alias(Str,"STRIDER",1) then
      return Dna_Strider;
    elsif Is_An_Alias(Str,"GCG",2) then
      return GCG;
    elsif Is_An_Alias(Str,"GENBANK",2) then
      return Genbank;
    else
      raise INVALID_SEQUENCE_FORMAT;
    end if;
  end Opt_Sequence_Format;

  function Opt_Sequence_Model(Str : in String) return Sequence_Model is
  begin
    if Is_An_Alias(Str,"DNA",1) then
      return DNA;
    elsif Is_An_Alias(Str,"ADN",3) then
      return DNA;
    elsif Is_An_Alias(Str,"RNA",1) then
      return RNA;
    elsif Is_An_Alias(Str,"ARN",3) then
      return RNA;
    elsif Is_An_Alias(Str,"Protein",1) then
      return Protein;
    else
      raise INVALID_SEQUENCE_MODEL;
    end if;
  end Opt_Sequence_Model;
  
  function Opt_Sequence_Strand(Str : in String) return Strand_Type is
  begin
    if Is_An_Alias(Str,"WATSON",1) then
      return Watson;
    elsif Is_An_Alias(Str,"CRICK",1) then
      return Crick;
    elsif Is_An_Alias(Str,"BOTH",1) then
      return BOTH;
    else
      raise INVALID_STRAND;
    end if;
  end Opt_Sequence_Strand;
    
end Options_Util;
