--------------------------------------------------------------------------
-- Rb_Tree                                  Copyrigth ENS ULM 1986-1997 --
-- Pierre VINCENS                   V 01.000                 16/04/1996 --
-- Pierre VINCENS                   V 01.001                 30/03/1998 --
--------------------------------------------------------------------------
with Ada.Unchecked_Deallocation;

package body Rb_Tree is


-------------------------------------------------------------------------
-- Quelques fonctions utilitaires
-------------------------------------------------------------------------
  procedure Free is new Ada.Unchecked_Deallocation(Tree_Node,Tree_Ptr);
  procedure Free_Entries is
    new Ada.Unchecked_Deallocation(Tree_Entries_Array,Tree_Entries);

  function ">"(A,B : in Element_Type) return Boolean;
  function "<="(A,B : in Element_Type) return Boolean;
  function ">="(A,B : in Element_Type) return Boolean;
  pragma Inline(">","<=",">=");

  procedure Rotate_Left(Tree  : in out Tree_Handler;
                        X     : in Tree_Ptr);
  procedure Rotate_Right(Tree  : in out Tree_Handler;
                         X     : in Tree_Ptr);
  --pragma Inline(Rotate_Left,Rotate_Right);

  procedure Insert_Fixup(Tree  : in out Tree_Handler;
                         Ptr   : in Tree_Ptr);
  procedure Delete_Fixup(Tree  : in out Tree_Handler;
                         Ptr   : in out Tree_Ptr);


  function ">"(A,B : in Element_Type) return Boolean is
  begin
    return not ((A < B) or (A = B));
  end ">";

  function "<="(A,B : in Element_Type) return Boolean is
  begin
    return A < B or A = B;
  end "<=";

  function ">="(A,B : in Element_Type) return Boolean is
  begin
    return not (A < B);
  end ">=";

  procedure Rotate_Left(Tree  : in out Tree_Handler;
                        X     : in Tree_Ptr) is
    Y : Tree_Ptr := X.Right;
    P : Tree_Ptr := NIL.Parent;
  begin
    -- construire le lien droit
    X.Right := Y.Left;
    Y.Left.Parent := X;
    -- construire le lien gauche
    Y.Parent := X.Parent;
    if X.Parent = null then
      Tree.Root := Y;
    else
      if X = X.Parent.Left then
        X.Parent.Left := Y;
      else
        X.Parent.Right := Y;
      end if;
    end if;
    -- relier X et Y
    Y.Left := X;
    X.Parent := Y;
    -- restaurer NIL
    NIL.Parent := P;
  end Rotate_Left;

  procedure Rotate_Right(Tree  : in out Tree_Handler;
                         X     : in Tree_Ptr) is
    Y : Tree_Ptr := X.Left;
    P : Tree_Ptr := NIL.Parent;
  begin
    -- construire le lien gauche
    X.Left := Y.Right;
    Y.Right.Parent := X;
    -- construire le lien droit
    Y.Parent := X.Parent;
    if X.Parent = null then
      Tree.Root := Y;
    else
      if X = X.Parent.Right then
        X.Parent.Right := Y;
      else
        X.Parent.Left := Y;
      end if;
    end if;
    -- relier X et Y
    Y.Right := X;
    X.Parent := Y;
    -- restaurer NIL
    NIL.Parent := P;
  end Rotate_Right;

  procedure Insert_Fixup(Tree  : in out Tree_Handler;
                         Ptr   : in Tree_Ptr) is
    X : Tree_Ptr := Ptr;
    Y : Tree_Ptr;
  begin
    while X /= Tree.Root and then X.Parent.Color = Red loop
      -- Violation de la regle des couleurs
      if X.Parent = X.Parent.Parent.Left then
        -- alors on traite le noeud droit
        Y := X.Parent.Parent.Right;
        if Y.Color = Red then
          -- l'oncle est rouge
          X.Parent.Color := Black;
          Y.Color := Black;
          X.Parent.Parent.Color := Red;
          X := X.Parent.Parent;
        else
          -- l'oncle est noir
          if X = X.Parent.Right then
            -- faire du fils droit un fils gauche
            X := X.Parent;
            Rotate_Left(Tree,X);
          end if;
          -- recolorier et retourner
          X.Parent.Color := Black;
          X.Parent.Parent.Color := Red;
          Rotate_Right(Tree,X.Parent.Parent);
        end if;
      else
        -- meme chose pour le noeud gauche
        Y := X.Parent.Parent.Left;
        if Y.Color = Red then
          -- l'oncle est rouge
          X.Parent.Color := Black;
          Y.Color := Black;
          X.Parent.Parent.Color := Red;
          X := X.Parent.Parent;
        else
          -- l'oncle est noir
          if X = X.Parent.Left then
            -- faire du fils gauche un fils droit
            X := X.Parent;
            Rotate_Right(Tree,X);
          end if;
          -- recolorier et retourner
          X.Parent.Color := Black;
          X.Parent.Parent.Color := Red;
          Rotate_Left(Tree,X.Parent.Parent);
        end if;
      end if;
    end loop;
    Tree.Root.Color := Black;
  end Insert_Fixup;

  procedure Insert(Tree    : in out Tree_Handler;
                   Element : in Element_Type) is
    Current : Tree_Ptr := Tree.Root;
    Ptr : Tree_Ptr := new Tree_Node'(NIL,NIL,null,Red,Element);
    Parent : Tree_Ptr := null;
  begin
    -- recherche la position ou doit etre ajoute l'element
    while Current /= NIL loop
      -- s'il existe deja un element identique, on ne l'ajoute pas
      if Element = Current.Element then Free(Ptr); return; end if;
      -- memoriser la position courante
      Parent := Current;
      -- parcourir l'arbre gauche ou droit en fonction de la valeur de
      -- l'element
      if Element < Current.Element then
        Current := Current.Left;
      else
        Current := Current.Right;
      end if;
    end loop;
    -- Ajouter le noeud dans l'arbre
    if Parent = null then
      -- le noeud est a jouter a la racine
      Tree.Root := Ptr;
    else
      if Element < Parent.Element then
        Parent.Left := Ptr;
      else
        Parent.Right := Ptr;
      end if;
      Ptr.Parent := Parent;
    end if;
    Insert_Fixup(Tree,Ptr);
    Tree.Count := Tree.Count + 1;
  end Insert;

  procedure Delete_Fixup(Tree  : in out Tree_Handler;
                         Ptr   : in out Tree_Ptr) is
    W : Tree_Ptr;
  begin
    while Ptr /= Tree.Root and then Ptr.Color = Black loop
      if Ptr = Ptr.Parent.Left then
        W := Ptr.Parent.Right;
        if W.Color = Red then
          W.Color := Black;
          Ptr.Parent.Color := Red;
          Rotate_Left(Tree,Ptr.Parent);
          W := Ptr.Parent.Right;
        end if;
        if W.Left.Color = Black and W.Right.Color = Black then
            W.Color := Red;
            Ptr := Ptr.Parent;
        else
          if W.Right.Color = Black then
            W.Left.Color := Black;
            W.Color := Red;
            Rotate_Right(Tree,W);
            W := Ptr.Parent.Right;
          end if;
          W.Color := Ptr.Parent.Color;
          Ptr.Parent.Color := Black;
          W.Right.Color := Black;
          Rotate_Left(Tree,Ptr.Parent);
          Ptr := Tree.Root;
        end if;
      else
        W := Ptr.Parent.Left;
        if W.Color = Red then
          W.Color := Black;
          Ptr.Parent.Color := Red;
          Rotate_Right(Tree,Ptr.Parent);
          W := Ptr.Parent.Left;
        end if;
        if W.Right.Color = Black and W.Left.Color = Black then
          W.Color := Red;
          Ptr := Ptr.Parent;
        else
          if W.Left.Color = Black then
            W.Right.Color := Black;
            W.Color := Red;
            Rotate_Left(Tree,W);
            W := Ptr.Parent.Left;
          end if;
          W.Color := Ptr.Parent.Color;
          Ptr.Parent.Color := Black;
          W.Left.Color := Black;
          Rotate_Right(Tree,Ptr.Parent);
          Ptr := Tree.Root;
        end if;
      end if;
    end loop;
    Ptr.Color := Black;
  end Delete_Fixup;

  procedure Delete(Tree    : in out Tree_Handler;
                   Address : in out Tree_Ptr) is
    X,Y    : Tree_Ptr;
    Ycolor : Nodecolor;
  begin
    if Address = null or Address = NIL then return; end if;
    -- trouver l'element sucesseur dans l'arbre
    if Address.Left = NIL or Address.Right = NIL then
      Y := Address;
    else
      Y := address.Right;
      while Y.Left /= NIL loop
        Y := Y.Left;
      end loop;
    end if;
    -- definir X comme le fils de Y
    if Y.Left = NIL then
      X := Y.Right;
    else
      X := Y.Left;
    end if;
    -- enlever Y de la chaine de parents
    X.Parent := Y.Parent;
    if Y.Parent = null then
      Tree.Root := X;
    else
      if Y = Y.Parent.Left then
        Y.Parent.Left := X;
      else
        Y.Parent.Right := X;
      end if;
    end if;
    -- Y est le noeud a enlever
    -- address est la donnee a enlever
    Ycolor := Y.Color;
    if Y = address then
      -- enlever Y=address
      Free(Y);
    else
      -- address et Y ne sont pas identique, remplacer address par Y
      Y.Left := address.Left;
      Y.Left.Parent := Y;
      Y.Right := address.Right;
      Y.Right.Parent := Y;
      Y.Parent := address.Parent;
      if Address.Parent = null then
        Tree.Root := Y;
      else
        if Address = Address.Parent.Left then
          Address.Parent.Left := Y;
        else
          Address.Parent.Right := Y;
        end if;
      end if;
      Y.Color := Address.Color;
      Free(Address);
    end if;
    if Ycolor = Black then
      Delete_Fixup(Tree,X);
    end if;
    Tree.Count := Tree.Count - 1;
    Address := null;
  end Delete;

  procedure Reset(Tree    : in out Tree_Handler) is
    procedure Clear_Tree(Node : in out Tree_Ptr) is
    begin
      if Node = NIL then
        return;
      end if;
      Clear_Tree(Node.Right);
      Clear_Tree(Node.Left);
      Free(Node);
    end Clear_Tree;
  begin
    Clear_Tree(Tree.Root);
    Tree.Root := NIL;
    Tree.Count := 0;
  end Reset;

  procedure Clean_And_Reset(Tree : in out Tree_Handler) is
    procedure Clear_Tree(Node : in out Tree_Ptr) is
    begin
      if Node = NIL then
        return;
      end if;
      Clear_Tree(Node.Right);
      Clear_Tree(Node.Left);
      Clean(Node.Element);
      Free(Node);
    end Clear_Tree;
  begin
    Clear_Tree(Tree.Root);
    Tree.Root := NIL;
    Tree.Count := 0;
  end Clean_And_Reset;

  function Find(Tree    : in Tree_Handler;
                Element : in Element_Type) return Tree_Ptr is
    Current : Tree_Ptr := Tree.Root;
  begin
    while Current /= NIL loop
      if Element = Current.Element then
        return Current;
      elsif Element < Current.Element then
        Current := Current.Left;
      else
        Current := Current.Right;
      end if;
    end loop;
    return null;
  end Find;
  
  function Find_Element(Tree      : in Tree_Handler;
                        Key       : in Data_Feature) return Tree_Ptr is
    Current : Tree_Ptr := Tree.Root;
  begin
    while Current /= NIL loop
      case Feature(Current.Element,Key) is
	when Less  => Current := Current.Left;
	when Equal => return Current;
	when More  => Current := Current.Right;
      end case;
    end loop;
    return null;
  end Find_Element;
  
  function Find_Between(Tree      : in Tree_Handler;
                        Condition : in Data_Feature;
                        Low,Hight : in Element_Type) return Tree_Ptr is
    function Check(X : in Tree_Ptr) return Tree_Ptr is
      Ptr : Tree_Ptr;
    begin
      if X /= NIL then
        if X.Element >= Low then
          Ptr := Check(X.Left);
          if Ptr /= null then return Ptr; end if;
        end if;
        if X.Element >= Low and X.Element <= Hight then
          if Feature(X.Element,Condition) then
            return X;
          end if;
        end if;
        if X.Element <= Hight then
          Ptr := Check(X.Right);
          if Ptr /= null then return Ptr; end if;
        end if;
      end if;
      return null;
    end Check;
  begin
    return Check(Tree.Root);
  end Find_Between;

  function Consult(Address : in Tree_Ptr) return Element_Type is
  begin
    return Address.Element;
  end Consult;

  procedure Apply(Tree : in Tree_Handler) is
    procedure Check(X : in Tree_Ptr) is
    begin
      if X /= NIL then
        Check(X.Left);
        Command(X.Element);
        Check(X.Right);
      end if;
    end Check;
  begin
    Check(Tree.Root);
  end Apply;

  procedure Apply_With_Data(Tree : in Tree_Handler;
                            Data : in out Data_Command) is
    procedure Check(X : in Tree_Ptr) is
    begin
      if X /= NIL then
        Check(X.Left);
        Command(X.Element,Data);
        Check(X.Right);
      end if;
    end Check;
  begin
    Check(Tree.Root);
  end Apply_With_Data;

  procedure Apply_Between(Tree : in Tree_Handler;
                          Data : in out Data_Command;
                          Low,Hight : in Element_Type) is
    procedure Check(X : in Tree_Ptr) is
    begin
      if X /= NIL then
        if X.Element >= Low then
          Check(X.Left);
        end if;
        if X.Element >= Low and X.Element <= Hight then
          Command(X.Element,Data);
        end if;
        if X.Element <= Hight then
          Check(X.Right);
        end if;
      end if;
    end Check;
  begin
    Check(Tree.Root);
  end Apply_Between;

  procedure Apply_Between_Until(Tree      : in Tree_Handler;
                                Data      : in out Data_Command;
                                Low,Hight : in Element_Type) is
    Find : Boolean := False;
    function Check(X : in Tree_Ptr) return Boolean is
    begin
      if X /= NIL then
        if X.Element >= Low then
          if Check(X.Left) then return True; end if;
        end if;
        Command(X.Element,Data);
        if X.Element >= Low and X.Element <= Hight then
          if Feature(X.Element,Data) then
            return True;
          end if;
        end if;
        if X.Element <= Hight then
          if Check(X.Right) then return True; end if;
        end if;
      end if;
      return False;
    end Check;
  begin
    Find := Check(Tree.Root);
  end Apply_Between_Until;

  procedure Apply_Between_If_To_First(Tree      : in Tree_Handler;
                                      Condition : in Data_Feature;
                                      Data      : in out Data_Command;
                                      Low,Hight : in Element_Type;
                                      Apply     : out Boolean) is
    function Check(X : in Tree_Ptr) return Boolean is
    begin
      if X /= NIL then
        if X.Element >= Low then
          if Check(X.Left) then return True; end if;
        end if;
        if X.Element >= Low and X.Element <= Hight then
          if Feature(X.Element,Condition) then
            Command(X.Element,Data);
            return True;
          end if;
        end if;
        if X.Element <= Hight then
          if Check(X.Right) then return True; end if;
        end if;
      end if;
      return False;
    end Check;
  begin
    Apply := Check(Tree.Root);
  end Apply_Between_If_To_First;

  function Is_Empty(Tree : in Tree_Handler) return Boolean is
  begin
    return Tree.Root = NIL;
  end Is_Empty;

  function Count(Tree : in Tree_Handler) return Natural is
  begin
    return Tree.Count;
  end Count;

  procedure Get_Entries(Tree : in Tree_Handler; 
			Entries : in out Tree_Entries) is
    I   : Natural;
    procedure Down(X : in Tree_Ptr) is
    begin
      if X /= NIL then
        Down(X.Left);
        Entries(I) := X;
	I := I + 1;
        Down(X.Right);
      end if;
    end Down;
  begin
    -- initier un tableau de la taille necessaire
    Entries := new Tree_Entries_Array(1..Count(Tree));
    I := Entries'First;
    -- parcourir l'arbre recursivement
    Down(Tree.Root);
  end Get_Entries;

  procedure Free(Entries : in out Tree_Entries) is
  begin
    Free_Entries(Entries);
  end Free;

begin
  NIL.Left := NIL;
  NIL.Right := NIL;
  NIL.Color := Black;
end Rb_Tree;
