--------------------------------------------------------------------------
-- Sequence_Alignment.Basic                 Copyrigth ENS ULM 1996-1999 --
-- Pierre VINCENS                   V 00.000                 02/06/1999 --
--------------------------------------------------------------------------
-- Paquetage issu du dcoupage de Sequence_Alignment pour faire de ce 
-- dernier un paquetage "Pure"
--------------------------------------------------------------------------
with Options_Util; use  Options_Util;
with Strlib; use  Strlib;
with Type_specifications; use Type_Specifications;

with Sequence_Specifications; use Sequence_Specifications;
with Sequence_Io; use Sequence_Io;
with Unchecked_Deallocation;

package Sequence_Alignment.Basic is


  type Alignment_Model is record
    -- identificateur des sequences
    Name1, Name2         : Varstring;
    Strand1, Strand2     : Strand_Type;
    -- Type des squences alignes
    Model                : Sequence_Model;
    -- methodes et paramtres utilises pour realiser l'Alignment
    Method               : Alignment_Method := Global_Alignment;
    Progression          : Progression_Method := DVH;
    Matrix               : Varstring;
    Gap_Weight           : Integer := -50;
    Gap_Length_Weight    : Integer :=  -3;
    Match_Weight         : Integer := 100;
    Mismatch_Weight      : Integer := -90;
    -- quantificateur de l'Alignment realise
    Length               : Integer     := 0;
    Nb_Gaps              : Integer     := 0;
    Gaps_Length          : Integer     := 0;
    Quality              : Integer     := 0;
    Quality_Mean         : Real        := 0.0;
    Quality_Std          : Real        := 0.0;
    Quality_Zscore       : Real        := 0.0;
    Quality_Sng          : Real        := 0.0;
    Ratio                : Real        := 0.0;
    Nb_Match             : Integer     := 0;
    Nb_Mismatch          : Integer     := 0;
    Similarity           : Percentage  := 0.0;
    Identity             : Percentage  := 0.0;
    Max_Continuous_Match : Integer     := 0;
    -- l'alignement obtenu et les positions de depart et de fin dans les
    -- squence initiales
    Seq1                 : Varstring;
    Start1,Stop1         : Integer;
    Seq2                 : Varstring;
    Start2,Stop2         : Integer;
  end record;

  type Alignment_Access is access Alignment_Model;
  type Alignments_Array is array(Integer range <>) of Alignment_Access;
  type Alignments_Access is access Alignments_Array;
  
  procedure Free(Alignment : in out Alignment_Access);
    -- Librer la place mmoire occupe par un alignement

  procedure Free(Alignments : in out Alignments_Access);
    -- Librer la place mmoire occupe par un tableau d'alignement

  function To_Alignment_Summary(Alignment: in Alignment_Model) 
				return Alignment_Summary_Model;
     -- Conversion vers le format summary

end Sequence_Alignment.Basic;
