--------------------------------------------------------------------------
-- Sequence_Alignment.Print                 Copyrigth ENS ULM 1996-2004 --
-- Pierre VINCENS                   V 00.000                 19/11/1997 --
-- Pierre VINCENS                   V 00.000                 01/03/2004 --
--------------------------------------------------------------------------
with Options_Util; use  Options_Util;
with Strlib; use Strlib;
with Strlib.IO; use  Strlib.IO;

with Type_specifications; use Type_Specifications;
with Type_Specifications.IO; use Type_Specifications.IO;
use  Type_Specifications.IO.Int_Io;
use  Type_Specifications.IO.Real_Io;
use  Type_Specifications.Real_Lib;

with Sequence_Specifications; use Sequence_Specifications;
with Sequence_Specifications.Basic; use Sequence_Specifications.Basic;
with Sequence_Io; use Sequence_Io;

package body Sequence_Alignment.Print is

  procedure Print_Alignment(File             : in Ada.Text_Io.File_Type;
                            Alignment        : in Alignment_Model;
                            Sequence_By_Line : in Integer := 50) is
    Points_By_Line : constant Integer := Sequence_By_Line / 10;
    Line_Length    : constant Integer := Points_By_Line * 10;
  begin
    -- affichage de l'en-tete
    Put_Line(File,"Sequence Alignment of: ");
    Set_Col(File,10);
    Put_line(File,To_String(Alignment.Name1) &
	     " (" & Strand_Type'Image(Alignment.Strand1) & ")");
    Set_Col(File,10);
    Put_line(File,To_String(Alignment.Name2) &
    	     " (" & Strand_Type'Image(Alignment.Strand2) & ")");
    New_Line(File,2);
    Put(File,"Method: ");
    case Alignment.Method is
      when Global_Alignment =>
        Put(File,"Global optimisation alignment (Needleman and Wunsch)");
      when Local_Alignment =>
        Put(File,"Local optimisation alignment (Smith and Waterman)");
    end case;
    case Alignment.Progression is
      when DHV => Put(File," - DHV");
      when DVH => Put(File," - DVH");
      when VDH => Put(File," - VDH");
      when VHD => Put(File," - VHD");
      when HDV => Put(File," - HDV");
      when HVD => Put(File," - HVD");
    end case;
    New_Line(File,2);
    Put_line(File,"Parameters: ");
    if Length(Alignment.Matrix) = 0 then
      Set_Col(File,10); Put(File,"Match Weight: ");
      Set_Col(File,30); Put(File,Alignment.Match_Weight,6);
      Set_Col(File,45); Put(File,"Mismatch Weight: ");
      Set_Col(File,65); Put(File,Alignment.Mismatch_Weight,6);
    else
      Set_Col(File,10); Put(File,"Score matrix: ");
      Set_Col(File,30); Put(File,To_String(Alignment.Matrix));
    end if;
    New_Line(File);
    Set_Col(File,10); Put(File,"Gap Weight: ");
    Set_Col(File,30); Put(File,Alignment.Gap_Weight,6);
    Set_Col(File,45); Put(File,"Gap Length Weight: ");
    Set_Col(File,65); Put(File,Alignment.Gap_Length_Weight,6);
    New_Line(File,2);
    Put_line(File,"Results: ");
    Set_Col(File,10); Put(File,"Quality: ");
    Set_Col(File,26); Put(File,Alignment.Quality,10);
    Set_Col(File,45); Put(File,"Ratio: ");
    Set_Col(File,65); Put(File,Alignment.Ratio,6,3,0);
    New_Line(File);
    Set_Col(File,10); Put(File,"Percent Similarity: ");
    Set_Col(File,30); Put(File,Alignment.Similarity,6,3,0);
    Set_Col(File,45); Put(File,"Percent Identity: ");
    Set_Col(File,65); Put(File,Alignment.Identity,6,3,0);
    New_Line(File);
    Set_Col(File,10); Put(File,"Nb Gaps: ");
    Set_Col(File,30); Put(File,Alignment.Nb_Gaps,6);
    Set_Col(File,45); Put(File,"Gaps Length: ");
    Set_Col(File,65); Put(File,Alignment.Gaps_Length,6);
    New_Line(File);
    Set_Col(File,10); Put(File,"Number of Match: ");
    Set_Col(File,30); Put(File,Alignment.Nb_Match,6);
    Set_Col(File,45); Put(File,"Number of Mismatch: ");
    Set_Col(File,65); Put(File,Alignment.Nb_Mismatch,6);
    New_Line(File);
    Set_Col(File,10); Put(File,"Length: ");
    Set_Col(File,30); Put(File,Alignment.Length,6);
    Set_Col(File,45); Put(File,"Continuous Matches: ");
    Set_Col(File,65); Put(File,Alignment.Max_Continuous_Match,6);
    if Alignment.Quality_Mean /= 0.0 and Alignment.Quality_Std /= 0.0 then      
      New_Line(File,2);
      Put_line(File,"Signifiance: ");
      Set_Col(File,10); Put(File,"Quality Mean: ");
      Set_Col(File,26); Put(File,Alignment.Quality_Mean,10,3,0);
      Set_Col(File,45); Put(File,"Quality Std: ");
      Set_Col(File,61); Put(File,Alignment.Quality_Std,10,3,0);
      New_Line(File);
      Set_Col(File,10); Put(File,"Quality Z-score: ");
      Set_Col(File,30); Put(File,Alignment.Quality_Zscore,6,3,0);
      Set_Col(File,45); Put(File,"Quality Signifiance: ");
      Set_Col(File,65); Put(File,Alignment.Quality_Sng,6,3,0);
    end if;
    New_Line(File,3);
    Put_line(File,"Alignment: ");
    -- Affichage de la sequence
    Print_Sequence: declare
      -- indice sur de debut et fin de ligne sur la sequence alignee
      Line_Start : Integer;
      Line_Stop  : Integer := 0;
      -- point de reference dans les deux squences
      Pos1    : Integer := Alignment.Start1;
      Pos2    : Integer := Alignment.Start2;
      -- direction de progression des deux squences
      Dir1,Dir2 : Integer := 1;
      -- l'acces aux sequences
      Seq1 : String_Access renames Alignment.Seq1.Value;
      Seq2 : String_Access renames Alignment.Seq2.Value;
      -- et la longueur totale de l'alignement y compris
      -- les extrmits non alignes
      Alignment_Length : Integer renames Alignment.Length;
    begin
      -- Regarder le sens de progression de la squence
      if Alignment.Start1 > Alignment.Stop1 then
	-- Il s'agit d'une squence reverse complmentaire
	Dir1 := -1;
      end if;
      if Alignment.Start2 > Alignment.Stop2 then
	-- Il s'agit d'une squence reverse complmentaire
	Dir2 := -1;
      end if;
      -- Afficher la squence ligne  ligne
      while Line_Stop < Alignment_Length loop
        -- calculer l'indice dans la sequence du dbut et de la fin de la ligne
        Line_Start := Line_Stop + 1;
        Line_Stop := Line_Stop + Line_Length;
        if Line_Stop > Alignment_Length then
          Line_Stop := Alignment_Length;
        end if;
        -- ecrire la ligne de points
        Set_Col(File,12);
        for I in 1..Points_By_Line loop
          Put(File,"         .");
        end loop;
        New_Line(File);
        -- ecrire la premiere sequence
        Put(File,Pos1,8);
        Set_Col(File,12);
        Put(File,Seq1(Line_Start..Line_Stop));
        Set_Col(File,Positive_Count(16+Points_By_Line*10));
	case Alignment.Model is
	  when DNA | RNA =>
	    for I in Line_Start..Line_Stop loop
	      if Is_A_Nucleotide(Seq1(I)) then
		Pos1 := Pos1 + Dir1;
	      end if;
	    end loop;
	  when Protein =>
	    for I in Line_Start..Line_Stop loop
	      if Is_An_Amino_Acid(Seq1(I)) then
		Pos1 := Pos1 + Dir1;
	      end if;
	    end loop;
	  when others =>
	    raise INVALID_SEQUENCE_MODEL;
	end case;
        Put(File,Pos1-Dir1,8);
        New_Line(File);
        -- ecrire la ligne intermediaire
        Set_Col(File,12);
        for I in Line_Start..Line_Stop loop
          case Alignment.Model is
	    when DNA | RNA =>
	      if ((Is_A_Nucleotide(Seq1(I)) and Seq2(I) = 'N') or
		  (Is_A_Nucleotide(Seq2(I)) and Seq1(I) = 'N')) then
		Put(File,':');
	      elsif Is_A_Nucleotide(Seq1(I)) and (Seq1(I) = Seq2(I)) then
		Put(File,'|');
	      else
		Put(File,' ');
	      end if;
	    when Protein =>
	      if ((Is_An_Amino_Acid(Seq1(I)) and Seq2(I) = 'X') or
		  (Is_An_Amino_Acid(Seq2(I)) and Seq1(I) = 'X')) then
		Put(File,':');
	      elsif Is_An_Amino_Acid(Seq1(I)) and (Seq1(I) = Seq2(I)) then
		Put(File,'|');
	      else
		Put(File,' ');
	      end if;
	    when others =>
	      raise INVALID_SEQUENCE_MODEL;
	  end case;
	end loop;
        New_Line(File);
        -- ecrire la deuxieme sequence
        Put(File,Pos2,8);
        Set_Col(File,12);
        Put(File,Seq2(Line_Start..Line_Stop));
        Set_Col(File,Positive_Count(16+Points_By_Line*10));
        for I in Line_Start..Line_Stop loop
          case Alignment.Model is
	    when DNA | RNA =>
	      if Is_A_Nucleotide(Seq2(I)) then
		Pos2 := Pos2 + Dir2;
	      end if;
	    when Protein =>
	      if Is_An_Amino_Acid(Seq2(I)) then
		Pos2 := Pos2 + Dir2;
	      end if;
	    when others =>
	      raise INVALID_SEQUENCE_MODEL;
	  end case;
        end loop;
        Put(File,Pos2-Dir2,8);
        New_Line(File);
      end loop;
    end Print_Sequence;
  end Print_Alignment;

  procedure Print_Summary(File      : in Ada.Text_Io.File_Type;
                          Alignment : in Alignment_Model) is
  begin
    Put_The_N_Last_Characters(File,To_String(Alignment.Name1),Summary_Id_Length);
    Put(File," " & Image(Alignment.Strand1));
    case Alignment.Strand1 is
      when Watson => 
	Put(File,Alignment.Start1,12);
	Put(File,Alignment.Stop1,12);
      when Crick => 
	Put(File,Alignment.Stop1,12);
    	Put(File,Alignment.Start1,12);
      when others =>
	raise Invalid_Strand;
    end case;
    Put(File,' ');
    Put_The_N_Last_Characters(File,To_String(Alignment.Name2),Summary_Id_Length);
    Put(File," " & Image(Alignment.Strand2));
    case Alignment.Strand2 is
      when Watson => 
	Put(File,Alignment.Start2,12);
	Put(File,Alignment.Stop2,12);
      when Crick => 
      	Put(File,Alignment.Stop2,12);
      	Put(File,Alignment.Start2,12);
      when others =>
	raise Invalid_Strand;
    end case;
    Put(File,' ');
    case Alignment.Method is
      when Global_Alignment =>
        Put(File,"G ");
      when Local_Alignment =>
        Put(File,"L ");
    end case;
    Put(File,Alignment.Quality,10);
    Put(File,Alignment.Ratio,4,4,0);
    Put(File,Alignment.Similarity,4,3,0);
    Put(File,Alignment.Identity,4,3,0);
    Put(File,Alignment.Length,8);
    Put(File,Alignment.Nb_Match,8);
    Put(File,Alignment.Nb_Mismatch,8);
    Put(File,Alignment.Nb_Gaps,5);
    Put(File,Alignment.Gaps_Length,8);
    Put(File,Alignment.Max_Continuous_Match,8);
    Put(File,Alignment.Quality_Mean,10,3,0);
    Put(File,Alignment.Quality_Std,10,3,0);
    Put(File,Alignment.Quality_Zscore,10,3,0);
    Put(File,Alignment.Quality_Sng,6,3,0);
  end Print_Summary;

  procedure Print_Summary(File      : in Ada.Text_Io.File_Type;
                          Alignment : in Alignment_Summary_Model) is
  begin
    Put(File,Alignment.Name1);
    Put(File," " & Image(Alignment.Strand1));
    case Alignment.Strand1 is
      when Watson => 
	Put(File,Alignment.Start1,12);
	Put(File,Alignment.Stop1,12);
      when Crick => 
	Put(File,Alignment.Stop1,12);
    	Put(File,Alignment.Start1,12);
      when others =>
	raise Invalid_Strand;
    end case;
    Put(File,' ');
    Put(File,Alignment.Name2);
    Put(File," " & Image(Alignment.Strand2));
    case Alignment.Strand2 is
      when Watson => 
	Put(File,Alignment.Start2,12);
	Put(File,Alignment.Stop2,12);
      when Crick => 
      	Put(File,Alignment.Stop2,12);
      	Put(File,Alignment.Start2,12);
      when others =>
	raise Invalid_Strand;
    end case;
    Put(File,' ');
    if Alignment.Length = 0 then
      -- L'alignement n'a pas t ralis
      Put(File,"NOT ALIGNED");
    else
      -- L'alignement est correct
      case Alignment.Method is
	when Global_Alignment =>
	  Put(File,"G ");
	when Local_Alignment =>
	  Put(File,"L ");
      end case;
      Put(File,Alignment.Quality,10);
      Put(File,Alignment.Ratio,4,4,0);
      Put(File,Alignment.Similarity,4,3,0);
      Put(File,Alignment.Identity,4,3,0);
      Put(File,Alignment.Length,8);
      Put(File,Alignment.Nb_Match,8);
      Put(File,Alignment.Nb_Mismatch,8);
      Put(File,Alignment.Nb_Gaps,5);
      Put(File,Alignment.Gaps_Length,8);
      Put(File,Alignment.Max_Continuous_Match,8);
      Put(File,Alignment.Quality_Mean,10,3,0);
      Put(File,Alignment.Quality_Std,10,3,0);
      Put(File,Alignment.Quality_Zscore,10,3,0);
      Put(File,Alignment.Quality_Sng,6,3,0);
    end if;
  end Print_Summary;

end Sequence_Alignment.Print;


