--------------------------------------------------------------------------
-- Single_List                              Copyrigth ENS ULM 1986-1997 --
-- Pierre VINCENS                   V 01.000                 16/04/1996 --
--------------------------------------------------------------------------
with Ada.Unchecked_Deallocation;

package body Single_List is

  procedure Free is new Ada.Unchecked_Deallocation(List_Node,List_Ptr);
  procedure Free_List is
    new Ada.Unchecked_Deallocation(List_Entries_Array,List_Entries);

  procedure Free(Entries : in out List_Entries) is
  begin
    Free_List(Entries);
  end Free;

  procedure Append(List    : in out List_Handler;
                   Element : in Element_Type) is
    Ptr : List_Ptr := new List_Node'(null,Element);
  Adr : List_Ptr := List.First;
  begin
    if List.First = null then
      -- la liste est vide
      List.First := Ptr;
      return;
    end if;
    while Adr.Next /= null loop
      -- adr est different de null
      Adr := Adr.Next;
    end loop;
    -- on a atteint le dernier element de la liste
    Adr.Next := Ptr;
  end Append;

  procedure Insert(List    : in out List_Handler;
                   Element : in Element_Type) is
    Ptr : List_Ptr := new List_Node'(List.First,Element);
  begin
    List.First := Ptr;
  end Insert;

  function Consult(Address : in List_Ptr) return Element_Type is
  begin
    return Address.Element;
  end Consult;

  procedure Delete(List    : in out List_Handler;
                   Address : in out List_Ptr) is
    Ptr : List_Ptr := List.First;
  begin
    if Address = List.First then
      -- l'element a supprimer est celui qui est en tete de la liste
      List.First := Address.Next;
    else
      -- l'element a supprimer est dans le milieu de la liste
      while Ptr.Next /= Address loop
        -- l'element suivant n'est pas l'element attendu
        if Ptr.Next = null then
          -- on a atteind la fin de la liste sans avoir trouve l'element
          -- peut etre vaudrait il mieux lever une exception
          return;
        end if;
        -- on avance dans la liste
        Ptr := Ptr.Next;
      end loop;
      -- on a trouve l'element
      Ptr.Next := Address.Next;
    end if;
    -- On supprime reellement l'element
    Free(Address);
  end Delete;

  procedure Modify(Address : in List_Ptr;
                   Data    : in out Data_Command) is
  begin
    Command(Address.Element,Data);
  end Modify;

  function Find_First_Element(List : in List_Handler) return List_Ptr is
    Ptr : List_Ptr := List.First;
  begin
    -- parcourir la liste
    while Ptr /= null loop
      if Feature(Ptr.Element) then
        return Ptr;
      end if;
      Ptr := Ptr.Next;
    end loop;
    return null;
  end Find_First_Element;

  function Find_Element(List : in List_Handler;
                        Data : in Data_Type) return List_Ptr is
    Ptr : List_Ptr := List.First;
  begin
    -- parcourir la liste
    while Ptr /= null loop
      if Feature(Ptr.Element,Data) then
        return Ptr;
      end if;
      Ptr := Ptr.Next;
    end loop;
    return null;
  end Find_Element;

  procedure Apply(List : in List_Handler) is
    Ptr,Next : List_Ptr := List.First;
  begin
    -- parcourir la liste
    while Ptr /= null loop
      Next := Ptr.Next;
      Command(Ptr.Element);
      Ptr := Next;
    end loop;
  end Apply;

  procedure Apply_With_Data(List : in List_Handler;
                            Data : in out Data_Command) is
    Ptr,Next : List_Ptr := List.First;
  begin
    -- parcourir la liste
    while Ptr /= null loop
      Next := Ptr.Next;
      Command(Ptr.Element,Data);
      Ptr := Next;
    end loop;
  end Apply_With_Data;

  procedure Apply_With_Address(List : in List_Handler) is
    Ptr,Next : List_Ptr := List.First;
  begin
    -- parcourir la liste
    while Ptr /= null loop
      Next := Ptr.Next;
      Command(Ptr.Element,Ptr);
      Ptr := Next;
    end loop;
  end Apply_With_Address;

  procedure Apply_If(List      : in List_Handler;
                     Condition : in Data_Feature;
                     Data      : in out Data_Command) is
    Ptr,Next : List_Ptr := List.First;
  begin
    -- parcourir la liste
    while Ptr /= null loop
      Next := Ptr.Next;
      if Feature(Ptr.Element,Condition) then
        Command(Ptr.Element,Data);
      end if;
      Ptr := Next;
    end loop;
  end Apply_If;

  procedure Apply_While(List      : in List_Handler;
                        Condition : in Data_Feature;
                        Data      : in out Data_Command) is
    Ptr,Next : List_Ptr := List.First;
  begin
    -- parcourir la liste
    while Ptr /= null loop
      Next := Ptr.Next;
      exit when not Feature(Ptr.Element,Condition);
      Command(Ptr.Element,Data);
      Ptr := Next;
    end loop;
  end Apply_While;

  procedure Apply_Until(List      : in List_Handler;
                        Data      : in out Data_Command) is
    Ptr,Next : List_Ptr := List.First;
  begin
    -- parcourir la liste
    while Ptr /= null loop
      Next := Ptr.Next;
      Command(Ptr.Element,Data);
      exit when Feature(Ptr.Element,Data);
      Ptr := Next;
    end loop;
  end Apply_Until;

  procedure Apply_If_To_First(List      : in List_Handler;
                              Condition : in Data_Feature;
                              Data      : in out Data_Command;
                              Apply     : out Boolean) is
    Ptr,Next : List_Ptr := List.First;
  begin
    -- parcourir la liste
    while Ptr /= null loop
      Next := Ptr.Next;
      if Feature(Ptr.Element,Condition) then
        Command(Ptr.Element,Data);
        Apply := True;
        return;
      end if;
      Ptr := Next;
    end loop;
    Apply := False;
  end Apply_If_To_First;

  procedure Apply_And_Remove_When(List : in out List_Handler;
                                  Data : in out Data_Type) is
    Ptr,Next : List_Ptr := List.First;
    Remove   : Boolean;
  begin
    -- parcourir la liste
    while Ptr /= null loop
      Next := Ptr.Next;
      Command(Ptr.Element,Data,Remove);
      if Remove then
        Delete(List,Ptr);
      end if;
      Ptr := Next;
    end loop;
  end Apply_And_Remove_When;

  procedure Reset(List : in out List_Handler) is
    Ptr,Next : List_Ptr := List.First;
  begin
    -- liberer la place occupee par la liste
    while Ptr /= null loop
      Next := Ptr.Next;
      Free(Ptr);
      Ptr := Next;
    end loop;
    -- mettre a jour la tete et la queue de la liste
    List.First := null;
  end Reset;

  procedure Clean_And_Reset(List : in out List_Handler) is
    Ptr,Next : List_Ptr := List.First;
  begin
    -- liberer la place occupee par la liste
    while Ptr /= null loop
      Next := Ptr.Next;
      Clean(Ptr.Element);
      Free(Ptr);
      Ptr := Next;
    end loop;
    -- mettre a jour la tete et la queue de la liste
    List.First := null;
  end Clean_And_Reset;

  function Is_Empty(List : in List_Handler) return Boolean is
  begin
    return List.First = null;
  end Is_Empty;

  function Count(List : in List_Handler) return Natural is
    Nb : Integer := 0;
    Adr : List_Ptr := List.First;
  begin
    while Adr /= null loop
      Adr := Adr.Next;
      Nb := Nb + 1;
    end loop;
    return Nb;
  end Count;

  procedure Get_Entries(List : in List_Handler;
                        Entries : in out List_Entries) is
    Ptr : List_Ptr := List.First;
    I   : Natural;
  begin
    -- initier un tableau de la taille necessaire
    Entries := new List_Entries_Array(1..Count(List));
    I := Entries'First;
    -- parcourir la liste
    while Ptr /= null loop
      Entries(I) := Ptr;
      I := I + 1;
      Ptr := Ptr.Next;
    end loop;
  end Get_Entries;

end Single_List;
