--------------------------------------------------------------------------
-- Static_Fifo                              Copyrigth ENS ULM 1998-1999 --
-- Pierre VINCENS                   V 01.000                 18/04/1998 --
-- Pierre VINCENS                   V 01.001                 28/12/1998 --
-- Pierre VINCENS                   V 01.002                 04/03/1999 --
--------------------------------------------------------------------------
-- Gestion d'une pile FIFO (First In First Out) sous la forme d'un tableau
-- La gestion est identique a celle du package FIFO dynamique. Les seules
-- differences sont dues a la limitation de la taille de la pile specifiee
-- lors de l'initialisation.
-- Ce gnrique doit etre instancie avec le type de donnees a stocker
--------------------------------------------------------------------------
-- V 01.001
-- Modifications mineures 
--   o Ajout de la fonction top
--   o Suppression du mode in out pour le paramtre stack dans la procedure 
--     top
--------------------------------------------------------------------------
-- V 01.002
-- Modifications mineures 
--   o Ajout de la fonction generique is_in
--------------------------------------------------------------------------
generic
  type Element_Type is private;
  Stack_Size : Positive := 200;
package Static_Fifo is

  type Stack_Handler is limited private;

  STACK_EMPTY     : exception;
  STACK_FULL      : exception;
  STACK_TOO_LARGE : exception;

  procedure Push(Stack : in out Stack_Handler; Element : in Element_Type);
  pragma Inline(Push);
  -- empiler un element
  
  procedure Pull(Stack : in out Stack_Handler; Element : out Element_Type);
  pragma Inline(Pull);
  -- depiler un element
  
  procedure Top(Stack : in Stack_Handler; Element : out Element_Type);
  pragma Inline(Top);
  -- recopier l'element en haut de la pile
  
  function Top(Stack : in Stack_Handler) return Element_Type;
  pragma Inline(Top);
  -- consulter l'element en haut de la pile
  
  generic
    with function Feature(A,B : in Element_Type) return Boolean;
  function Is_In(Stack : in Stack_Handler;
		 Value : in Element_Type) return Boolean;
  pragma Inline(Is_In);
  -- applique la fonction feature  chaque lment de la pile
  -- et retourne vrai  la premire occurence trouv pour laquelle
  -- feature retourne vrai. La principale application est de vrifier
  -- si un lment est dj dans la pile
    
  procedure Resize(Stack : in out Stack_Handler; Size : in Positive);
  pragma Inline(Resize);
  -- modifier la taille de la pile
  
  procedure Reset(Stack : in out Stack_Handler);
  pragma Inline(Reset);
  -- reinitialiser la pile
  
  generic
    with procedure Clean(Element : in out Element_Type);
  procedure Clean_And_Reset(Stack : in out Stack_Handler);
  pragma Inline(Reset);
  -- reinitialiser la pile apres avoir applique la procedure Clean sur chaque
  -- element avant sa destruction
  
  function Is_Empty(Stack : in Stack_Handler) return Boolean;
  pragma Inline(Is_Empty);
  function Is_Not_Empty(Stack : in Stack_Handler) return Boolean;
  pragma Inline(Is_Not_Empty);
  -- tester si la pile est vide ou non vide
  
  function Is_Full(Stack : in Stack_Handler) return Boolean;
  pragma Inline(Is_Full);
  function Is_Not_Full(Stack : in Stack_Handler) return Boolean;
  pragma Inline(Is_Not_Full);
  -- tester si la pile est pleine ou non pleine
  
  function Count(Stack : in Stack_Handler) return Natural;
  pragma Inline(Count);
  -- compte le nombre d'elements present de la pile.

private

  type Stack_Array is array(Natural range <>) of Element_Type;
  type Stack_Access is access Stack_Array;
  type Stack_Handler is record
    Data       : Stack_Access;
    Top,Bottom : Natural := 0;
    Size       : Positive := Stack_Size;
    Count      : Natural := 0;
  end record;

end Static_Fifo;
  

