--------------------------------------------------------------------------
-- Type_Specifications                      Copyrigth ENS ULM 1986-1997 --
-- Pierre VINCENS                   V 01.000                 06/11/1997 --
--------------------------------------------------------------------------
--
-- Ce paquetage definit des types d'usage gnral et quelques
-- fonctionnalits de conversion
-- Les instanciations des entres-sorties sur ces types sont faites
-- dans le paquetage fils type_specifications.IO
--
--------------------------------------------------------------------------
with System;
with Ada.Numerics.Generic_Elementary_Functions;

package Type_Specifications is

  pragma Pure(Type_Specifications);

  -- Gestion des noms de fichiers
  Max_Length_Filename : constant Natural :=  255;
  Max_Length_Filepath : constant Natural := 1024;
  Max_Length_Pathname : constant Natural := 1024;

  subtype Filename is String;
  subtype Filepath is String;
  subtype Pathname is String;

  -- Le type octet
  type Byte is range 0..255;
  for Byte'Size use 1 * SYSTEM.STORAGE_UNIT;

  -- Les types entiers

  -- WARNING: le type integer est defini d'une maniere variable en fonction
  -- du host et du compilateur (ex: 16 bits sur sun3/alsys, 32 bits sur
  -- sun4/telesoft.)
  -- Les types natural et positives sont definis a partir du type integer.
  -- Ces types sont utilises en particulier pour la definition
  -- du type string.

  type Integer_8  is range -2**07..2**07-1;
  type Integer_16 is range -2**15..2**15-1;
  type Integer_32 is range -2**31..2**31-1;
  for Integer_8'Size  use 1 * SYSTEM.STORAGE_UNIT;
  for Integer_16'Size use 2 * SYSTEM.STORAGE_UNIT;
  for Integer_32'Size use 4 * SYSTEM.STORAGE_UNIT;

  subtype Short_Integer is Integer_8;
  subtype Long_Integer  is Integer_32;

  -- Les types reels
  type Real is digits 6;
  type Long_Real is digits 15;
  for Real'Size use 4 * SYSTEM.STORAGE_UNIT;
  for Long_Real'Size use 8 * SYSTEM.STORAGE_UNIT;

  subtype Degre is Real;
  subtype Radian is Real;

  -- Fonctions de service
  function Degre_To_Radian (X : in Degre) return Radian;
  function Radian_To_Degre (X : in Radian) return Degre;

  -- Instanciations des librairies mathmatiques
  package Real_Lib is new Ada.Numerics.Generic_Elementary_Functions(Real);
  package Long_Real_Lib is new
    Ada.Numerics.Generic_Elementary_Functions(Long_Real);

end Type_Specifications;
