--------------------------------------------------------------------------
-- Yaap_Options                             Copyrigth ENS ULM 1996-1999 --
-- Pierre VINCENS                   V 01.000                 17/03/1999 --
-- Pierre VINCENS                   V 01.001                 16/02/2000 --
--------------------------------------------------------------------------
with Ada.Text_Io; use  Ada.Text_Io;
with Ada.Command_Line; use Ada.Command_Line;
with Ada.Exceptions; use Ada.Exceptions;

with Type_specifications; use Type_specifications;
with Type_Specifications.IO; use Type_Specifications.IO;
use  Type_Specifications.IO.Int_Io;
use  Type_Specifications.IO.Real_Io;
use  Type_Specifications.Real_Lib;

with Strlib; use  Strlib;
with Options_Util; use  Options_Util;

with Sequence_Specifications; use Sequence_Specifications;
with Sequence_Specifications.Basic; use Sequence_Specifications.Basic;
with Sequence_Io; use  Sequence_Io;

with Sequence_Alignment; use Sequence_Alignment;
with Sequence_Alignment.Print; use Sequence_Alignment.Print;
with Sequence_Alignment.Global; use Sequence_Alignment.Global;
with Sequence_Alignment.Local; use Sequence_Alignment.Local;

with Yaap_Version; use Yaap_Version;

package body Yaap_Options is

  procedure Get_Options(Options : in out Options_Type) is
    Count : Positive := 1;
    type Options_List is (Mismatch_Weight);
    type Options_Controlled_Array is array (Options_List) of Boolean;
    Options_Controlled : Options_Controlled_Array := (others => False);
  begin
    if Argument_Count = 0 then
      raise ARGUMENT_ERROR;
    end if;
    while Count <= Argument_Count loop
      if Is_An_Alias(Argument(Count),"-GLOBAL",3) then
        -- Utiliser la mthode d'alignement global (Needleman et Wunsch)
        Options.Method := Global_Alignment;
      elsif Is_An_Alias(Argument(Count),"-LOCAL",3) then
        -- Utiliser la mthode d'alignement local (Smith et Waterman)
        Options.Method := Local_Alignment;
      elsif Is_An_Alias(Argument(Count),"-BUilding",3) then
        -- lire la mthode d'optimisation
        Count := Count + 1;
        if Is_An_Alias(Argument(Count),"One",1) then
	  Options.Building := One;
	elsif Is_An_Alias(Argument(Count),"All_first",1) then
	  Options.Building := All_Firsts;
	elsif Is_An_Alias(Argument(Count),"Full",1) then
	  Options.Building := Full;
	else
	  Raise_Exception(Invalid_Data'Identity,"O_Building");
	end if;
      elsif Is_An_Alias(Argument(Count),"-OPtimize",3) then
        -- lire la mthode d'optimisation
        Count := Count + 1;
        if Is_An_Alias(Argument(Count),"No",1) then
	  Options.Optimize := No;
	elsif Is_An_Alias(Argument(Count),"Short",1) then
	  Options.Optimize := Short;
	elsif Is_An_Alias(Argument(Count),"Large",1) then
	  Options.Optimize := Large;
	else
	  Raise_Exception(Invalid_Data'Identity,"O_Optimize");
	end if;
      elsif Is_An_Alias(Argument(Count),"-BAckward",3) then
        -- lire la mthode d'optimisation
        Count := Count + 1;
        if Is_An_Alias(Argument(Count),"DVH",2) then
	  Options.Progression := DVH;
	elsif Is_An_Alias(Argument(Count),"DHV",2) then
	  Options.Progression := DHV;
	elsif Is_An_Alias(Argument(Count),"VDH",2) then
	  Options.Progression := VDH;
	elsif Is_An_Alias(Argument(Count),"VHD",2) then
	  Options.Progression := VHD;
	elsif Is_An_Alias(Argument(Count),"HDV",2) then
	  Options.Progression := HDV;
	elsif Is_An_Alias(Argument(Count),"HVD",2) then
	  Options.Progression := HVD;
	else
	  Raise_Exception(Invalid_Data'Identity,"O_Backward");
	end if;
      elsif Is_An_Alias(Argument(Count),"-S1",3) then
        -- lire le nom de la sequence 1
        Count := Count + 1;
        Copy(Argument(count),Options.Name1);
      elsif Is_An_Alias(Argument(Count),"-I1",3) then
        -- lire l'identificateur de la sequence 1
        Count := Count + 1;
        Copy(Argument(count),Options.Identifier1);
      elsif Is_An_Alias(Argument(Count),"-F1",3) then
        -- definir le format de lecture de la sequence 1
        Count := Count + 1;
        begin
	  Options.Format1 := Opt_Sequence_Format(Argument(Count));
	exception
	  when Invalid_Sequence_Format =>
	    Raise_Exception(Invalid_Sequence_Format'Identity,"F1");
        end;
      elsif Is_An_Alias(Argument(Count),"-T1",3) then
        -- definir le type de la sequence 1
        Count := Count + 1;
	begin
	  Options.Model1 := Opt_Sequence_Model(Argument(Count));
	exception
	  when Invalid_Sequence_Model =>
	    Raise_Exception(Invalid_Sequence_Model'Identity,"T1");
        end;
      elsif Is_An_Alias(Argument(Count),"-B1",3) then
        -- definir le debut pour la sequence 1
        Count := Count + 1;
	begin
	  Options.begin1 := Opt_Int(Argument(Count));
	exception
          when Data_Error => 
            Raise_Exception(Invalid_Data'Identity,"O_Start 1");
        end;
      elsif Is_An_Alias(Argument(Count),"-E1",3) then
        -- definir la fin pour la sequence 1
        Count := Count + 1;
	begin
	  Options.end1 := Opt_Int(Argument(Count));
	exception
          when Data_Error => 
            Raise_Exception(Invalid_Data'Identity,"O_Stop 1");
        end;
      elsif Is_An_Alias(Argument(Count),"-R1",3) then
        -- indique que l'on inverse la sequence 1
        Options.inverse1 := True;
      elsif Is_An_Alias(Argument(Count),"-C1",3) then
        -- indique que l'on prend la sequence complementaire
        Options.Complementaire1 := True;
      elsif Is_An_Alias(Argument(Count),"-ST1",4) then
        -- indique sur quel brin doit porter l'analyse
        Count := Count + 1;
        begin
          Options.Strand1 := Opt_Sequence_Strand(Argument(Count));
	  if Options.Strand1 = Both then
	    raise Invalid_Strand;
	  end if;
        exception
          when Invalid_Strand => 
            Raise_Exception(Invalid_Strand'Identity,"1");
        end;
      elsif Is_An_Alias(Argument(Count),"-S2",3) then
        -- lire le nom de la sequence 2
        Count := Count + 1;
        Copy(Argument(count),Options.Name2);
      elsif Is_An_Alias(Argument(Count),"-I2",3) then
        -- lire l'identificateur de la sequence 2
        Count := Count + 1;
        Copy(Argument(count),Options.Identifier2);
      elsif Is_An_Alias(Argument(Count),"-F2",3) then
        -- definir le format de lecture de la sequence 2
        Count := Count + 1;
	begin
	  Options.Format2 := Opt_Sequence_Format(Argument(Count));
	exception
	  when Invalid_Sequence_Format =>
	    Raise_Exception(Invalid_Sequence_Format'Identity,"F2");
        end;
      elsif Is_An_Alias(Argument(Count),"-T2",3) then
        -- definir le type de la sequence 2
        Count := Count + 1;
	begin
	  Options.Model2 := Opt_Sequence_Model(Argument(Count));
	exception
	  when Invalid_Sequence_Model =>
	    Raise_Exception(Invalid_Sequence_Model'Identity,"T2");
        end;
      elsif Is_An_Alias(Argument(Count),"-B2",3) then
        -- definir le debut pour la sequence 2
        Count := Count + 1;
	begin
	  Options.begin2 := Opt_Int(Argument(Count));
	exception
          when Data_Error => 
            Raise_Exception(Invalid_Data'Identity,"O_Start 2");
        end;
      elsif Is_An_Alias(Argument(Count),"-E2",3) then
        -- definir la fin pour la sequence 2
        Count := Count + 1;
	begin
	  Options.End2 := Opt_Int(Argument(Count));
	exception
          when Data_Error => 
            Raise_Exception(Invalid_Data'Identity,"O_Stop 2");
        end;
      elsif Is_An_Alias(Argument(Count),"-R2",3) then
        -- indique que l'on inverse la sequence 2
        Options.inverse2 := True;
      elsif Is_An_Alias(Argument(Count),"-C2",3) then
        -- indique que l'on prend la sequence complementaire
        Options.Complementaire2 := True;
      elsif Is_An_Alias(Argument(Count),"-ST2",4) then
        -- indique sur quel brin doit porter l'analyse
        Count := Count + 1;
        begin
          Options.Strand2 := Opt_Sequence_Strand(Argument(Count));
	  if Options.Strand2 = Both then
	    raise Invalid_Strand;
	  end if;
        exception
          when Invalid_Strand => 
            Raise_Exception(Invalid_Strand'Identity,"2");
        end;
      elsif Is_An_Alias(Argument(Count),"-SCORE_MATRIX",3) then
        -- lire le nom de la matrice d'appariement  utiliser
        Count := Count + 1;
        Copy(Argument(count),Options.Matrix);
      elsif Is_An_Alias(Argument(Count),"-MATCH_WEIGHT",3)  or
	Is_An_Alias(Argument(Count),"-MW",3) then
        -- Poids affect  un match
        Count := Count + 1;
	begin
	  Options.Match_Weight := Opt_Int(Argument(Count));
	exception
          when Data_Error => 
            Raise_Exception(Invalid_Data'Identity,"O_Match_Weight");
        end;
      elsif Is_An_Alias(Argument(Count),"-MISMATCH_WEIGHT",3) then
        -- Poids affect  un mismatch
        Count := Count + 1;
	begin
	  Options_Controlled(Mismatch_Weight) := True;
	  Options.Mismatch_Weight := Opt_Int(Argument(Count));
	exception
          when Data_Error => 
            Raise_Exception(Invalid_Data'Identity,"O_Mismatch_Weight");
        end;
      elsif Is_An_Alias(Argument(Count),"-GAP_WEIGHT",6) or
	Is_An_Alias(Argument(Count),"-GW",3) then
        -- Poids affect  la cration dn nouveau gap
        Count := Count + 1;
	begin
	  Options.Gap_Weight := Opt_Int(Argument(Count));
	exception
          when Data_Error => 
            Raise_Exception(Invalid_Data'Identity,"O_Gap_Weight");
        end;
      elsif Is_An_Alias(Argument(Count),"-GAP_LENGTH_WEIGHT",6) or
	Is_An_Alias(Argument(Count),"-LW",3) then
        -- Poids affect  l'ajout d'un lment dans un gap existant
        Count := Count + 1;
	begin
	  Options.Gap_Length_Weight := Opt_Int(Argument(Count));
	exception
          when Data_Error => 
            Raise_Exception(Invalid_Data'Identity,"O_Gap_Length_Weight");
        end;
      elsif Is_An_Alias(Argument(Count),"-SIMILARITY_THRESHOLD",13) or
	Is_An_Alias(Argument(Count),"-SIT",4) then
	-- Seuil pour prendre en compte une similitude entre deux rsidus
        Count := Count + 1;
	begin
	  Options.Similarity_Threshold := Opt_Int(Argument(Count));
	exception
          when Data_Error => 
            Raise_Exception(Invalid_Data'Identity,"O_Similarity_threshold");
        end;
      elsif Is_An_Alias(Argument(Count),"-RANDomization",5) or
	Is_An_Alias(Argument(Count),"-RND",4) then
	-- Pour dterminer le seuil de signification d'un alignement
        Count := Count + 1;
	begin
	  Options.Randomization := Opt_Int(Argument(Count));
	exception
          when Data_Error => 
            Raise_Exception(Invalid_Data'Identity,"O_Randomization");
        end;
      elsif Is_An_Alias(Argument(Count),"-PREserve",4) then
	-- Dans le cas de squence nucliques pour dfinir si l'on
	-- prserve la frquence nuclotidique, di, tri-nuclotidique,... 
        Count := Count + 1;
	begin
	  Options.Preserve := Opt_Int(Argument(Count));
	exception
          when Data_Error => 
            Raise_Exception(Invalid_Data'Identity,"O_Preserve");
        end;
      elsif Is_An_Alias(Argument(Count),"-END_WEIGHT",7) or
	Is_An_Alias(Argument(Count),"-EW",3)then
        -- indique que l'on doit tenir compte des gaps extrmits
	-- dans l'alignement global
        Options.With_End_Weight := True;
      elsif Is_An_Alias(Argument(Count),"-RESULTS",3) then
        -- indique que l'on affiche les resultats d'alignement dans un fichier
        Options.Print_Result := True;
      elsif Is_An_Alias(Argument(Count),"-NOSUMMARY",2) then
        -- indique que l'on n'affiche pas les couples a l'ecran
        Options.Summary := False;
      elsif Is_An_Alias(Argument(Count),"-SUMMARY",3) then
        -- indique que l'on affiche les couples dans un fichier
        Options.Print_Summary := True;
      elsif Is_An_Alias(Argument(Count),"-LENGTHMAX",2) then
        -- indique que l'on passe une liste de position a traiter
        Count := Count + 1;
	begin
	  Options.Max_Length := Opt_Int(Argument(Count));
	  if Options.Max_Length < 0 then raise Data_Error; end if; 
	exception
          when Data_Error => 
            Raise_Exception(Invalid_Data'Identity,"O_Length_Max");
        end;
      elsif Is_An_Alias(Argument(Count),"-LIST",3) then
        -- indique que l'on passe une liste de squences  considrer
	-- Cela implique obligatoirement le passage d'une option poslist
        Count := Count + 1;
        Copy(Argument(count),Options.Listfile);
      elsif Is_An_Alias(Argument(Count),"-POSITIONS",2) then
        -- indique que l'on passe une liste de position a traiter
        Options.Poslist := True;
        Count := Count + 1;
        Copy(Argument(count),Options.Posfile);
      elsif Is_An_Alias(Argument(Count),"-ADJUST",2) then
        -- indique que l'on veut tendre la zone  aligner au del
        -- des valeurs passes
        Count := Count + 1;
	begin
	  Options.Adjust := Opt_Int(Argument(Count));
	exception
          when Data_Error => 
            Raise_Exception(Invalid_Data'Identity,"O_Adjust");
        end;
      elsif Is_An_Alias(Argument(Count),"-LMIN",2) then
        -- indique la longueur minimale de l'alignement pour
	-- l'enregistrer dans le fichier
        Count := Count + 1;
	begin
	  Options.Lmin := Opt_Int(Argument(Count));
	exception
          when Data_Error => 
            Raise_Exception(Invalid_Data'Identity,"O_Lmin");
        end;
      elsif Is_An_Alias(Argument(Count),"-SMIN",2) then
        -- indique le pourcentage d'identit minimal de l'alignement pour
	-- l'enregistrer dans le fichier
        Count := Count + 1;
	begin
	  Options.Smin := Percentage(Opt_Real(Argument(Count)));
	  if Options.Smin < 0.0 or Options.Smin > 100.0 then
	    Raise_Exception(Invalid_Data'Identity,"O_Smin");
	  end if;
	exception
          when Data_Error => 
            Raise_Exception(Invalid_Data'Identity,"O_Smin");
        end;
      elsif Is_An_Alias(Argument(Count),"-VERBOSE",2) then
        -- indique que l'on affiche l'avancee de l'alignement
	-- On affiche tout de suite le numro de version
        Put_Line("Yaap version " & Version_Id & " - Copyright ENS Ulm");
	Options.Interactif := True;
      else
        raise ARGUMENT_ERROR;
      end if;
      Count := Count + 1;
    end loop;

    -- Test de validit des options
    if Length(Options.Listfile) = 0 then
      -- Les squences doivent avoir t donn sur la ligne de commande
      if Length(Options.Name1) = 0 then
	Raise_Exception(Invalid_Data'Identity,"O_Name 1");
      end if;
      if Length(Options.Name2) = 0 then
	Raise_Exception(Invalid_Data'Identity,"O_Name 2");
      end if;
      -- Si les identifieurs n'ont pas t indiqus, on les considre
      -- gaux aux racines des noms des squences
      if Length(Options.Identifier1) = 0 then
	Copy(Root_Name(Options.Name1),Options.Identifier1);
      end if;
      if Length(Options.Identifier2) = 0 then
	Copy(Root_Name(Options.Name2),Options.Identifier2);
      end if;
    else
      -- Les squences sont donnes dans une liste
      if Length(Options.Name1) /= 0 then
	Raise_Exception(Invalid_Data'Identity,"L_Name 1");
      end if;
      if Length(Options.Name2) /= 0 then
	Raise_Exception(Invalid_Data'Identity,"L_Name 2");
      end if;
      if not Options.Poslist then
	Raise_Exception(Invalid_Data'Identity,"L_Poslist");
      end if;
    end if;

    if not Options_Controlled(Mismatch_Weight) then
      -- La valeur par dfaut dpend de la mthode
      case Options.Method is
	when Global_Alignment => Options.Mismatch_Weight :=  0;
      	when Local_Alignment  => Options.Mismatch_Weight := -9;
      end case;
    end if;
  end Get_Options;

  procedure Print_Options is
  begin
    Put_Line("Yaap - " & Version_Id & ": argument error");
    Put_Line("  syntax is: yaap [-GLobal] ");
    Put_Line("                  [-LOcal] ");
    Put_Line("                  [-Building <One|All_first|Full>]");
    Put_Line("                  [-OPtimize <No|Short|large>]");
    Put_Line("                  [-BAckward <DVH|DHV|VDH|VHD|HDV|HVD>]");
    Put_Line("                  [-B1 start] [-E1 stop]");
    Put_Line("                  [-B2 start] [-E2 stop]");
    Put_Line("                  [-F1 <Ascii|GCg|GEnbank|Strider>]");
    Put_Line("                  [-F2 <Ascii|GCg|GEnbank|Strider>]");
    Put_Line("                  [-T1 <Dna|Rna|Protein>]");
    Put_Line("                  [-T2 <Dna|Rna|Protein>]");
    Put_Line("                  [-R1] [-C1]");
    Put_Line("                  [-R2] [-C2]");
    Put_Line("                  [-ST1 Watson|Crick]");
    Put_Line("                  [-ST2 Watson|Crick]");
    Put_Line("                  [-SCore_Matrix name]");
    Put_Line("                  [-MAtch_weight value]");
    Put_Line("                  [-MIsmatch_weight value]");
    Put_Line("                  [-GAP_Weight value] | [-GW value]");
    Put_Line("                  [-GAP_Length_weight value] | [-LW value]");
    Put_Line("                  [-RANDomization value] | [-RND value]");
    Put_Line("                  [-PREserve value]");
    Put_Line("                  [-END_WEight] | [-EW]");
    Put_Line("                  [-SIMILARITY_Threshold value] | [-SIT value]");
    Put_Line("                  [-Adjust value]");
    Put_Line("                  [-LEngthmax value]");
    Put_Line("                  [-LMin value]");
    Put_Line("                  [-SMin value]");
    Put_Line("                  [-LIstfile file]");
    Put_Line("                  [-Positions file]");
    Put_Line("                  [-REsults]");
    Put_Line("                  [-SUmmary]");
    Put_Line("                  [-Nosummary]");
    Put_Line("                  [-Verbose]");
    Put_Line("                  [-S1 sequence_file]");
    Put_Line("                  [-S2 sequence_file]");
    New_Line;
  end Print_Options;

end Yaap_Options;
